@@ -29,12 +29,11 @@
 #include "dvdauthor.h"
 #include "da-internal.h"
 
-static const char RCSID[]="$Id: //depot/dvdauthor/src/dvdvob.c#55 $";
 
 
 struct colorremap {
     int newcolors[16];
-    int state,curoffs,maxlen,nextoffs,skip;
+    int state,curoffs,maxlen,nextoffs,skip,ln_ctli;
     struct colorinfo *origmap;
 };
 
@@ -53,12 +52,20 @@ static int writefile=-1;
 
 static unsigned char videoslidebuf[15]={255,255,255,255, 255,255,255, 0,0,0,0, 0,0,0,0};
 
+
+/* The following are variants for the ways I've seen DVD's encoded */
+
+// Grosse Pointe Blank uses exactly 1/2 second for the FF/REW records
+// Bullitt uses 1/2 video second (i.e. in NTSC, 45045 PTS)
+#define DVD_FFREW_HALFSEC 45000
+// #define DVD_FFREW_HALFSEC (getratedenom(va)>>1)
+
 static pts_t calcpts(struct vobgroup *va,int cancomplain,int *didcomplain,pts_t *align,pts_t basepts,int nfields)
 {
     // I assume pts should round down?  That seems to be how mplex deals with it
     // also see later comment
 
-    int fpts=getframepts(va);
+    pts_t fpts=getframepts(va);
     int bpframe=(basepts*2-*align+fpts/2)/fpts;
     if( (*align+bpframe*fpts)/2 != basepts ) {
         if( !*didcomplain ) {
@@ -66,7 +73,7 @@ static pts_t calcpts(struct vobgroup *va,int cancomplain,int *didcomplain,pts_t
                 fprintf(stderr,"WARN: Video PTS does not line up on a multiple of a field.\n");
             *didcomplain=1;
         }
-        *align=basepts;
+        *align=basepts*2;
     } else
         nfields += bpframe;
     return (*align+nfields*fpts)/2;
@@ -283,9 +290,11 @@ static int mpa_len(unsigned char *b)
 static void writeflush()
 {
     if( !writebufpos ) return;
-    if( write(writefile,bigwritebuf,writebufpos) != writebufpos ) {
-        fprintf(stderr,"ERR:  Error writing data\n");
-        exit(1);
+    if( writefile!=-1 ) {
+        if( write(writefile,bigwritebuf,writebufpos) != writebufpos ) {
+            fprintf(stderr,"ERR:  Error writing data\n");
+            exit(1);
+        }
     }
     writebufpos=0;
 }
@@ -374,17 +383,22 @@ static void scanvideoptr(struct vobgroup *va,unsigned char *buf,struct vobuinfo
 
             vobgroup_set_video_framerate(va,frame);
             switch(frame) {
-            case 1:
-            case 4:
-            case 7:
+            case 1: // 24000/1001
+            case 4: // 30000/1001
+            case 7: // 60000/1001
                 vobgroup_set_video_attr(va,VIDEO_FORMAT,"ntsc");
                 break;
 
-            case 3:
-            case 6:
+            case 3: // 25
+            case 6: // 50
                 vobgroup_set_video_attr(va,VIDEO_FORMAT,"pal");
                 break;
 
+            case 2: // 24
+            case 5: // 30
+            case 8: // 60
+                break; // these are nonstandard, but at least we know what they are
+
             default:
                 fprintf(stderr,"WARN: unknown frame rate %d\n",frame);
                 break;
@@ -564,49 +578,11 @@ static void printpts(pts_t pts)
     fprintf(stderr,"%d.%03d",(int)(pts/90000),(int)((pts/90)%1000));
 }
 
-static FILE *openvob(char *f,int *ispipe)
-{
-    FILE *h;
-    int l=strlen(f);
-
-    if( l>0 && f[l-1]=='|' ) {
-        char *str;
-        int i;
-
-        f[l-1]=0;
-        str=(char *)malloc(l*2+1+10);
-        strcpy(str,"sh -c \"");
-        l=strlen(str);
-        for( i=0; f[i]; i++ ) {
-            if( f[i]=='\"' || f[i]=='\'' )
-                str[l++]='\\';
-            str[l++]=f[i];
-        }
-        str[l]=0;
-        strcpy(str+l,"\"");
-        h=popen(str,"r");
-        free(str);
-        ispipe[0]=1;
-    } else if( !strcmp(f,"-") ) {
-        h=stdin;
-        ispipe[0]=2;
-    } else if( f[0]=='&' && isdigit(f[1]) ) {
-        h=fdopen(atoi(&f[1]),"rb");
-        ispipe[0]=0;
-    } else {
-        h=fopen(f,"rb");
-        ispipe[0]=0;
-    }
-    if( !h ) {
-        fprintf(stderr,"ERR:  Error opening %s: %s\n",f,strerror(errno));
-        exit(1);
-    }
-    return h;
-}
-
 enum { CR_BEGIN0,   CR_BEGIN1,    CR_BEGIN2,    CR_BEGIN3, CR_SKIP0,
        CR_SKIP1,    CR_NEXTOFFS0, CR_NEXTOFFS1, CR_WAIT,   CR_CMD,
-       CR_SKIPWAIT, CR_COL0,      CR_COL1};
+       CR_SKIPWAIT, CR_COL0,      CR_COL1,      CR_CHGARG, CR_CHGLN0,
+       CR_CHGLN1,   CR_CHGLN2,    CR_CHGLN3,    CR_CHGPX0, CR_CHGPX1,
+       CR_CHGPX2 };
 
 static char *readpstr(char *b,int *i)
 {
@@ -692,9 +668,11 @@ static void procremap(struct colorremap *cr,unsigned char *b,int l,pts_t *timesp
             case 4: cr->skip=2; cr->state=CR_SKIPWAIT; break;
             case 5: cr->skip=6; cr->state=CR_SKIPWAIT; break;
             case 6: cr->skip=4; cr->state=CR_SKIPWAIT; break;
+            case 7: cr->skip=2; cr->state=CR_CHGARG; break;
             case 255: cr->state=CR_WAIT; break;
             default:
-	        break;
+                fprintf(stderr,"ERR: procremap encountered unknown subtitle command: %d\n",*b);
+                exit(1);
             }
             break;
 
@@ -708,6 +686,44 @@ static void procremap(struct colorremap *cr,unsigned char *b,int l,pts_t *timesp
             cr->state=CR_CMD;
             break;
            
+        case CR_CHGARG:
+	    if (!--cr->skip)
+                cr->state=CR_CHGLN0;
+	    break;
+
+        case CR_CHGLN0: cr->ln_ctli =b[0]<<24; cr->state=CR_CHGLN1; break;
+        case CR_CHGLN1: cr->ln_ctli|=b[0]<<16; cr->state=CR_CHGLN2; break;
+        case CR_CHGLN2: cr->ln_ctli|=b[0]<< 8; cr->state=CR_CHGLN3; break;
+
+        case CR_CHGLN3:
+	    cr->ln_ctli|=b[0];
+            
+	    if (cr->ln_ctli==0x0fffffff)
+                cr->state=CR_CMD;
+	    else {
+                cr->ln_ctli>>=12;
+                cr->ln_ctli&=0xf;
+                cr->skip=2;
+                cr->state=CR_CHGPX0;
+	    }
+	    break;
+
+        case CR_CHGPX0:
+	    if (!--cr->skip) { cr->skip=2; cr->state=CR_CHGPX1; }
+	    break;
+
+	case CR_CHGPX1:
+            remapbyte(cr,b);
+	    if (!--cr->skip) { cr->skip=2; cr->state=CR_CHGPX2; }
+	    break;
+
+        case CR_CHGPX2:
+	    if (!--cr->skip) {
+                if (!--cr->ln_ctli) cr->state=CR_CHGLN0;
+                else { cr->skip=2; cr->state=CR_CHGPX0; }
+	    }
+	    break;
+
         default:
             assert(0);
         }
@@ -730,7 +746,7 @@ static void printvobustatus(struct vobgroup *va,int cursect)
 
 static void audio_scan_ac3(struct audchannel *ach,unsigned char *buf,int sof,int len)
 {
-    u_int32_t parse;
+    uint32_t parse;
     int acmod,lfeon,nch=0;
     char attr[4];
 
@@ -808,9 +824,8 @@ static void audio_scan_pcm(struct audchannel *ach,unsigned char *buf,int len)
 int FindVobus(char *fbase,struct vobgroup *va,int ismenu)
 {
     unsigned char *buf;
-    FILE *h;
     int cursect=0,fsect=-1,vnum,outnum=-ismenu+1;
-    int ispipe,vobid=0;
+    int vobid=0;
     struct mp2info {
         int hdrptr;
         unsigned char buf[6];
@@ -825,6 +840,7 @@ int FindVobus(char *fbase,struct vobgroup *va,int ismenu)
         struct vob *s=va->vobs[vnum];
         int prevvidsect=-1;
         struct vscani vsi;
+        struct vfile vf;
 
         vsi.lastrefsect=0;
         for( i=0; i<32; i++ )
@@ -835,7 +851,11 @@ int FindVobus(char *fbase,struct vobgroup *va,int ismenu)
         vsi.firstgop=1;
 
         fprintf(stderr,"\nSTAT: Processing %s...\n",s->fname);
-        h=openvob(s->fname,&ispipe);
+        vf=varied_open(s->fname,O_RDONLY);
+        if( !vf.h ) {
+            fprintf(stderr,"ERR:  Error opening %s: %s\n",s->fname,strerror(errno));
+//            exit(1);
+        }
         memset(mp2hdr,0,8*sizeof(struct mp2info));
         while(1) {
             if( fsect == 524272 ) {
@@ -849,7 +869,7 @@ int FindVobus(char *fbase,struct vobgroup *va,int ismenu)
             }
             buf=writegrabbuf();
 
-            i=fread(buf,1,2048,h);
+            i=fread(buf,1,2048,vf.h);
             if( i!=2048 ) {
                 if( i==-1 ) {
                     fprintf(stderr,"\nERR:  Error while reading: %s\n",strerror(errno));
@@ -912,28 +932,39 @@ int FindVobus(char *fbase,struct vobgroup *va,int ismenu)
                             i+=2;
                             for( j=0; j<k; j++ ) {
                                 struct button *b;
-                                char *st=readpstr(buf,&i);
+                                struct buttoninfo *bi,bitmp;
+                                char *bn=readpstr(buf,&i);
                                     
-                                if( !findbutton(s->p,st,0) ) {
-                                    fprintf(stderr,"ERR:  Cannot find button '%s' as referenced by the subtitle\n",st);
+                                if( !findbutton(s->p,bn,0) ) {
+                                    fprintf(stderr,"ERR:  Cannot find button '%s' as referenced by the subtitle\n",bn);
                                     exit(1);
                                 }
-                                b=&s->p->buttons[findbutton(s->p,st,0)-1];
+                                b=&s->p->buttons[findbutton(s->p,bn,0)-1];
+                                free(bn);
+
+                                if( b->numstream>=MAXBUTTONSTREAM ) {
+                                    fprintf(stderr,"WARN: Too many button streams; ignoring buttons\n");
+                                    bi=&bitmp;
+                                } else {
+                                    bi=&b->stream[b->numstream++];
+                                }
+
+                                bi->st=st;
 
                                 i+=2; // skip modifier
-                                b->autoaction=buf[i++];
-                                if( !b->autoaction ) {
-                                    b->grp=buf[i];
-                                    b->x1=read2(buf+i+1);
-                                    b->y1=read2(buf+i+3);
-                                    b->x2=read2(buf+i+5);
-                                    b->y2=read2(buf+i+7);
+                                bi->autoaction=buf[i++];
+                                if( !bi->autoaction ) {
+                                    bi->grp=buf[i];
+                                    bi->x1=read2(buf+i+1);
+                                    bi->y1=read2(buf+i+3);
+                                    bi->x2=read2(buf+i+5);
+                                    bi->y2=read2(buf+i+7);
                                     i+=9;
                                     // up down left right
-                                    b->up=readpstr(buf,&i);
-                                    b->down=readpstr(buf,&i);
-                                    b->left=readpstr(buf,&i);
-                                    b->right=readpstr(buf,&i);
+                                    bi->up=readpstr(buf,&i);
+                                    bi->down=readpstr(buf,&i);
+                                    bi->left=readpstr(buf,&i);
+                                    bi->right=readpstr(buf,&i);
                                 }
                             }
                             break;
@@ -957,14 +988,9 @@ int FindVobus(char *fbase,struct vobgroup *va,int ismenu)
             }
             if( buf[0]==0 && buf[1]==0 && buf[2]==1 && buf[3]==0xba ) {
                 pts_t newscr=readscr(buf+4);
-                if((newscr==0)&&(lastscr>0))
-                {
-                  backoffs-=lastscr;
-                  fprintf(stderr,"\nWARN:  SCR reset. New back offset = %d \n",backoffs);
-                }
-                else if( newscr < lastscr ) {
-//                fprintf(stderr,"ERR:  SCR moves backwards, remultiplex input. %d<%d\n",newscr,lastscr);
-//                 exit(1);
+                if( newscr < lastscr ) {
+                    fprintf(stderr,"ERR:  SCR moves backwards, remultiplex input.\n");
+//                    exit(1);
                 }
                 lastscr=newscr;
                 if( !hadfirstvobu )
@@ -974,11 +1000,13 @@ int FindVobus(char *fbase,struct vobgroup *va,int ismenu)
             if( fsect == -1 ) {
                 char newname[200];
                 fsect=0;
-                if( outnum>=0 )
-                    sprintf(newname,"%s_%d.VOB",fbase,outnum);
-                else
-                    strcpy(newname,fbase);
-                writeopen(newname);
+                if( fbase ) {
+                    if( outnum>=0 )
+                        sprintf(newname,"%s_%d.VOB",fbase,outnum);
+                    else
+                        strcpy(newname,fbase);
+                    writeopen(newname);
+                }
             }
             if( buf[14] == 0 &&
                 buf[15] == 0 &&
@@ -1018,8 +1046,7 @@ int FindVobus(char *fbase,struct vobgroup *va,int ismenu)
                     vi->numref=0;
                     vi->hasseqend=0;
                     vi->hasvideo=0;
-                    vi->sectdata=(unsigned char *)malloc(2048);
-                    memcpy(s->vi[s->numvobus-1].sectdata,buf,2048);
+                    memcpy(s->vi[s->numvobus-1].sectdata,buf,0x26); // save pack and system header; the rest will be reconstructed later
                     if( !(s->numvobus&15) )
                         printvobustatus(va,cursect);
                     vsi.lastrefsect=0;
@@ -1074,13 +1101,10 @@ int FindVobus(char *fbase,struct vobgroup *va,int ismenu)
                 buf[14] == 0 &&
                 buf[15] == 0 &&
                 buf[16] == 1 &&
-                ((buf[17] & 0xf8) == 0xc0 || buf[17]==0xbd) &&
-                buf[21] & 128 ) {
-                pts_t pts0=readpts(buf+23),pts1,backpts1;
+                ((buf[17] & 0xf8) == 0xc0 || buf[17]==0xbd)) {
+                pts_t pts0=0,pts1=0,backpts1=0;
                 int dptr=buf[22]+23,endop=read2(buf+18)+20;
-                int audch;
-                pts1=pts0;
-                backpts1=0;
+                int audch,haspts=(buf[21]&128);
                 if( buf[17]==0xbd ) {
                     int sid=buf[dptr],offs=read2(buf+dptr+2);
 
@@ -1132,11 +1156,17 @@ int FindVobus(char *fbase,struct vobgroup *va,int ismenu)
                     memcpy(mp2hdr[index].buf,buf+dptr+len-3,3);
                     audiodesc_set_audio_attr(&s->audch[audch].ad,&s->audch[audch].adwarn,AUDIO_SAMPLERATE,"48khz");
                 }
-                // fprintf(stderr,"aud ch=%d pts %d - %d\n",audch,pts0,pts1);
+                if( haspts ) {
+                    pts0=readpts(buf+23);
+                    pts1+=pts0;
+                } else if( pts1>0 ) {
+                    fprintf(stderr,"WARN: Audio channel %d contains sync headers but has no PTS.\n",audch);
+                }
+                // fprintf(stderr,"aud ch=%d pts %d - %d (%d)\n",audch,pts0,pts1,pts1-pts0);
                 // fprintf(stderr,"pts[%d] %d (%02x %02x %02x %02x %02x)\n",va->numaudpts,pts,buf[23],buf[24],buf[25],buf[26],buf[27]);
                 if( audch<0 || audch>=64 ) {
                     fprintf(stderr,"WARN: Invalid audio channel %d\n",audch);
-                } else {
+                } else if( haspts ) {
                     struct audchannel *ach=&s->audch[audch];
 
                     if( ach->numaudpts>=ach->maxaudpts ) {
@@ -1159,9 +1189,10 @@ int FindVobus(char *fbase,struct vobgroup *va,int ismenu)
                         if( ach->audpts[ach->numaudpts-1].pts[1]<pts0 ) {
                             if( audch>=32 )
                                 goto noshow;
-                            fprintf(stderr,"WARN: Discontinuity in audio channel %d; please remultiplex input.\n",audch);
+                            fprintf(stderr,"WARN: Discontinuity of %" PRId64" in audio channel %d; please remultiplex input.\n",pts0-ach->audpts[ach->numaudpts-1].pts[1],audch);
+                            // fprintf(stderr,"last=%d, this=%d\n",ach->audpts[ach->numaudpts-1].pts[1],pts0);
                         } else if( ach->audpts[ach->numaudpts-1].pts[1]>pts0 )
-                            fprintf(stderr,"WARN: Audio pts for channel %d moves backwards; please remultiplex input.\n",audch);
+                            fprintf(stderr,"WARN: Audio pts for channel %d moves backwards by %" PRId64 "; please remultiplex input.\n",audch,ach->audpts[ach->numaudpts-1].pts[1]-pts0);
                         else
                             goto noshow;
                         fprintf(stderr,"WARN: Previous sector: ");
@@ -1202,21 +1233,7 @@ int FindVobus(char *fbase,struct vobgroup *va,int ismenu)
             cursect++;
             fsect++;
         }
-        switch(ispipe) {
-        case 0:
-            if (fclose(h)) {
-                fprintf(stderr,"\nERR:  Error reading from file: %s\n",strerror(errno));
-                exit(1);
-            }
-            break;
-        case 1:
-            if (pclose(h)) {
-                fprintf(stderr,"\nERR:  Error reading from pipe: %s\n",strerror(errno));
-                exit(1);
-            }
-            break;
-        case 2: break;
-        }
+        varied_close(vf);
         if( s->numvobus ) {
             int i;
             pts_t finalaudiopts;
@@ -1239,7 +1256,7 @@ int FindVobus(char *fbase,struct vobgroup *va,int ismenu)
             for( i=0; i<3; i++ ) {
                 pts_t pts_align=-1;
                 int complain=0, j;
-                
+
                 for( j=0; j<s->numvobus; j++ ) {
                     struct vobuinfo *vi=s->vi+j;
                     
@@ -1264,7 +1281,10 @@ int FindVobus(char *fbase,struct vobgroup *va,int ismenu)
                             break;
 
                         vi->sectpts[0]=vi->videopts[0];
-                        vi->sectpts[1]=vi->videopts[1];
+                        if( j+1 == s->numvobus && finalaudiopts > vi->videopts[1] )
+                            vi->sectpts[1]=finalaudiopts;
+                        else
+                            vi->sectpts[1]=vi->videopts[1];
                     }
                 }
                 if( !complain )
@@ -1317,7 +1337,10 @@ int FindVobus(char *fbase,struct vobgroup *va,int ismenu)
                         p=s->vi[i+1].sectpts[0];
                     else // the next vobu is an audio only vobu, and will backfill the pts as necessary
                         continue;
-                    assert(p>vi->sectpts[0]);
+                    if( p<=vi->sectpts[0] ) {
+                        fprintf(stderr, "ERR:  Audio and video are too poorly synchronised; you must remultiplex.\n");
+                        exit(1);
+                    }
                     vi->sectpts[1]=p;
                 }
             }
@@ -1393,7 +1416,7 @@ void MarkChapters(struct vobgroup *va)
 
                 if( lastcellid!=v &&
                     s->vob->vi[v].firstIfield!=0) {
-                    fprintf(stderr,"WARN: GOP is not closed on cell %d of source %s of pgc %d\n",k+1,s->fname,i+1);
+                    fprintf(stderr,"WARN: GOP may not be closed on cell %d of source %s of pgc %d\n",k+1,s->fname,i+1);
                 }
 
                 if( s->cells[k].endpts>=0 ) {
@@ -1473,10 +1496,10 @@ void MarkChapters(struct vobgroup *va)
         }
 }
 
-static pts_t getcellaudiopts(struct vobgroup *va,int vcid,int ach,int w)
+static pts_t getcellaudiopts(const struct vobgroup *va,int vcid,int ach,int w)
 {
-    struct vob *v=va->vobs[(vcid>>8)-1];
-    struct audchannel *a=&v->audch[ach];
+    const struct vob *v=va->vobs[(vcid>>8)-1];
+    const struct audchannel *a=&v->audch[ach];
     int ai=0;
 
     assert((vcid&255)==(w?v->numcells:1));
@@ -1485,19 +1508,19 @@ static pts_t getcellaudiopts(struct vobgroup *va,int vcid,int ach,int w)
     return a->audpts[ai].pts[w];
 }
 
-static int hasaudio(struct vobgroup *va,int vcid,int ach,int w)
+static int hasaudio(const struct vobgroup *va,int vcid,int ach,int w)
 {
-    struct vob *v=va->vobs[(vcid>>8)-1];
-    struct audchannel *a=&v->audch[ach];
+    const struct vob *v=va->vobs[(vcid>>8)-1];
+    const struct audchannel *a=&v->audch[ach];
 
     assert((vcid&255)==(w?v->numcells:1));
 
     return a->numaudpts!=0;
 }
 
-static pts_t getcellvideopts(struct vobgroup *va,int vcid,int w)
+static pts_t getcellvideopts(const struct vobgroup *va,int vcid,int w)
 {
-    struct vob *v=va->vobs[(vcid>>8)-1];
+    const struct vob *v=va->vobs[(vcid>>8)-1];
     int vi=0;
 
     assert((vcid&255)==(w?v->numcells:1));
@@ -1509,12 +1532,12 @@ static pts_t getcellvideopts(struct vobgroup *va,int vcid,int w)
     return v->vi[vi].sectpts[w];
 }
 
-static pts_t calcaudiodiff(struct vobgroup *va,int vcid,int ach,int w)
+static pts_t calcaudiodiff(const struct vobgroup *va,int vcid,int ach,int w)
 {
     return getcellvideopts(va,vcid,w)-getcellaudiopts(va,vcid,ach,w);
 }
 
-int calcaudiogap(struct vobgroup *va,int vcid0,int vcid1,int ach)
+int calcaudiogap(const struct vobgroup *va,int vcid0,int vcid1,int ach)
 {
     if( vcid0==-1 || vcid1==-1 )
         return 0;
@@ -1537,17 +1560,16 @@ int calcaudiogap(struct vobgroup *va,int vcid0,int vcid1,int ach)
         g2=calcaudiodiff(va,vcid1,ach,0);
         return g1!=g2;
     }
-    fprintf(stderr,"WARN: Do not know how to compute the audio gap, assuming discontinuity.\n");
+    fprintf(stderr,"WARN: Do not know how to compute the audio gap between '%s' and '%s', assuming discontinuity.\n",va->vobs[(vcid0>>8)-1]->fname,va->vobs[(vcid1>>8)-1]->fname);
     return 1;
 }
 
-void FixVobus(char *fbase,struct vobgroup *va,int ismenu)
+void FixVobus(char *fbase,const struct vobgroup *va,const struct workset *ws,int ismenu)
 {
     int h=-1;
     int i,j,pn,fnum=-2;
     pts_t scr;
     int vff,vrew,totvob,curvob;
-    unsigned char *buf;
 
     totvob=0;
     for( pn=0; pn<va->numvobs; pn++ )
@@ -1559,29 +1581,37 @@ void FixVobus(char *fbase,struct vobgroup *va,int ismenu)
 
         for( i=0; i<p->numvobus; i++ ) {
             struct vobuinfo *vi=&p->vi[i];
+            static unsigned char buf[2048];
 
             if( vi->fnum!=fnum ) {
                 char fname[200];
                 if( h >= 0 )
                     close(h);
                 fnum=vi->fnum;
-                if( fnum==-1 )
-                    strcpy(fname,fbase);
-                else
-                    sprintf(fname,"%s_%d.VOB",fbase,fnum);
-                h=open(fname,O_WRONLY|O_BINARY);
-                if( h < 0 ) {
-                    fprintf(stderr,"\nERR:  Error opening %s: %s\n",fname,strerror(errno));
-                    exit(1);
+                if( fbase ) {
+                    if( fnum==-1 )
+                        strcpy(fname,fbase);
+                    else
+                        sprintf(fname,"%s_%d.VOB",fbase,fnum);
+                    h=open(fname,O_WRONLY|O_BINARY);
+                    if( h < 0 ) {
+                        fprintf(stderr,"\nERR:  Error opening %s: %s\n",fname,strerror(errno));
+                        exit(1);
+                    }
                 }
             }
-            buf=vi->sectdata;
+            memcpy(buf,vi->sectdata,0x26);
+            write4(buf+0x26,0x1bf); // private stream 2
+            write2(buf+0x2a,0x3d4); // length
+            buf[0x2c]=0;
             memset(buf+0x2d,0,0x400-0x2d);
+            write4(buf+0x400,0x1bf); // private stream 2
+            write2(buf+0x404,0x3fa); // length
+            buf[0x406]=1;
             memset(buf+0x407,0,0x7ff-0x407);
 
             scr=readscr(buf+4);
 
-            buf[0x2c]=0;
             write4(buf+0x2d,vi->sector);
             write4(buf+0x39,vi->sectpts[0]); // vobu_s_ptm
             write4(buf+0x3d,vi->sectpts[1]); // vobu_e_ptm
@@ -1591,40 +1621,80 @@ void FixVobus(char *fbase,struct vobgroup *va,int ismenu)
                 
             if( p->p->numbuttons ) {
                 struct pgc *pg=p->p;
+                int mask=getsubpmask(&va->vd),ng,grp;
+                char idmap[3];
 
                 write2(buf+0x8d,1);
                 write4(buf+0x8f,p->vi[0].sectpts[0]);
                 write4(buf+0x93,-1);
                 write4(buf+0x97,-1);
-                write2(buf+0x9b,0x1000);
+
+                ng=0;
+                write2(buf+0x9b,0);
+                for( j=0; j<4; j++ )
+                    if( mask&(1<<j) ) {
+                        assert(ng<3);
+                        idmap[ng]=j;
+                        write2(buf+0x9b,read2(buf+0x9b)+0x1000+(((1<<j)>>1)<<(2-ng)*4));
+                        ng++;
+                    }
+                assert(ng>0);
+
                 buf[0x9e]=pg->numbuttons;
                 buf[0x9f]=pg->numbuttons;
                 memcpy(buf+0xa3,p->buttoncoli,24);
-                for( j=0; j<pg->numbuttons; j++ ) {
-                    struct button *b=pg->buttons+j;
-                    if( b->autoaction ) {
-                        buf[0xbb+j*18+3]=64;
-                    } else {
-                        buf[0xbb+j*18+0]=(b->grp*64)|(b->x1>>4);
-                        buf[0xbb+j*18+1]=(b->x1<<4)|(b->x2>>8);
-                        buf[0xbb+j*18+2]=b->x2;
-                        buf[0xbb+j*18+3]=(b->y1>>4);
-                        buf[0xbb+j*18+4]=(b->y1<<4)|(b->y2>>8);
-                        buf[0xbb+j*18+5]=b->y2;
-                        buf[0xbb+j*18+6]=findbutton(pg,b->up,(j==0)?pg->numbuttons:j);
-                        buf[0xbb+j*18+7]=findbutton(pg,b->down,(j+1==pg->numbuttons)?1:j+2);
-                        buf[0xbb+j*18+8]=findbutton(pg,b->left,(j==0)?pg->numbuttons:j);
-                        buf[0xbb+j*18+9]=findbutton(pg,b->right,(j+1==pg->numbuttons)?1:j+2);
+                for( grp=0; grp<ng; grp++ ) {
+                    unsigned char *boffs=buf+0xbb+18*(grp*36/ng);
+                    int sid=pg->subpmap[0][(int)idmap[grp]]&127;
+
+                    for( j=0; j<pg->numbuttons; j++ ) {
+                        static unsigned char compilebuf[128*8],*rbuf;
+                        struct button *b=pg->buttons+j;
+                        struct buttoninfo *bi;
+                        int k;
+
+                        for( k=0; k<b->numstream; k++ )
+                            if( b->stream[k].st==sid )
+                                break;
+                        if( k==b->numstream )
+                            continue;
+                        bi=&b->stream[k];
+
+                        if( bi->autoaction ) {
+                            boffs[3]=64;
+                        } else {
+                            boffs[0]=(bi->grp*64)|(bi->x1>>4);
+                            boffs[1]=(bi->x1<<4)|(bi->x2>>8);
+                            boffs[2]=bi->x2;
+                            boffs[3]=(bi->y1>>4);
+                            boffs[4]=(bi->y1<<4)|(bi->y2>>8);
+                            boffs[5]=bi->y2;
+                            boffs[6]=findbutton(pg,bi->up,(j==0)?pg->numbuttons:j);
+                            boffs[7]=findbutton(pg,bi->down,(j+1==pg->numbuttons)?1:j+2);
+                            boffs[8]=findbutton(pg,bi->left,(j==0)?pg->numbuttons:j);
+                            boffs[9]=findbutton(pg,bi->right,(j+1==pg->numbuttons)?1:j+2);
+                        }
+                        rbuf=vm_compile(compilebuf,compilebuf,ws,pg->pgcgroup,pg,b->cs,ismenu);
+                        if( rbuf-compilebuf==8 ) {
+                            memcpy(boffs+10,compilebuf,8);
+                        } else if( allowallreg ) {
+                            fprintf(stderr,"ERR:  Button command is too complex to fit in one instruction, and allgprm==true.\n");
+                            exit(1);
+                        } else
+                            write8(boffs+10,0x71,0x01,0x00,0x0F,0x00,j+1,0x00,0x0d); // g[15]=j && linktailpgc
+                        boffs+=18;
                     }
-                    write8(buf+0xbb+j*18+10,0x71,0x01,0x00,0x0F,0x00,j+1,0x00,0x0d); // g[0]=j && linktailpgc
                 }
             }
 
-            buf[0x406]=1;
             write4(buf+0x407,scr);
             write4(buf+0x40b,vi->sector); // current lbn
-            for( j=0; j<vi->numref; j++ )
-                write4(buf+0x413+j*4,vi->lastrefsect[j]-vi->sector);
+            if( vi->numref>0 ) {
+                for( j=0; j<vi->numref; j++ )
+                    write4(buf+0x413+j*4,vi->lastrefsect[j]-vi->sector);
+                for( ; j<3; j++ )
+                    write4(buf+0x413+j*4,vi->lastrefsect[vi->numref-1]-vi->sector);
+            }
             write2(buf+0x41f,vi->vobcellid>>8);
             buf[0x422]=vi->vobcellid;
             write4(buf+0x423,read4(buf+0x45));
@@ -1691,13 +1761,15 @@ void FixVobus(char *fbase,struct vobgroup *va,int ismenu)
             for( j=0; j<19; j++ ) {
                 int nff,nrew;
 
-                nff=findvobu(p,vi->sectpts[0]+timeline[j]*45000,
+                nff=findvobu(p,vi->sectpts[0]+timeline[j]*DVD_FFREW_HALFSEC,
                              vi->firstvobuincell,
                              vi->lastvobuincell);
                 // a hack -- the last vobu in the cell shouldn't have any forward ptrs
-                if( i==vi->lastvobuincell )
-                    nff=i+1;
-                nrew=findvobu(p,vi->sectpts[0]-timeline[j]*45000,
+                // EXCEPT this hack violates both Grosse Pointe Blank and Bullitt -- what was I thinking?
+                // if( i==vi->lastvobuincell )
+                // nff=i+1;
+
+                nrew=findvobu(p,vi->sectpts[0]-timeline[j]*DVD_FFREW_HALFSEC,
                               vi->firstvobuincell,
                               vi->lastvobuincell);
                 write4(buf+0x53d-j*4,getsect(p,i,nff,j>=15 && nff>vff+1,0x3fffffff));
@@ -1706,14 +1778,16 @@ void FixVobus(char *fbase,struct vobgroup *va,int ismenu)
                 vrew=nrew;
             }
 
-            lseek(h,vi->fsect*2048,SEEK_SET);
-            write(h,buf,2048);
+            if( h!=-1 ) {
+                lseek(h,vi->fsect*2048,SEEK_SET);
+                write(h,buf,2048);
+            }
             curvob++;
             if( !(curvob&15) ) 
                 fprintf(stderr,"STAT: fixing VOBU at %dMB (%d/%d, %d%%)\r",vi->sector/512,curvob+1,totvob,curvob*100/totvob);
         }
     }
-    if( h>=0 )
+    if( h!=-1 )
         close(h);
     if( totvob>0 )
         fprintf(stderr,"STAT: fixed %d VOBUS                         ",totvob);
