@@ -25,6 +25,18 @@ COMMENTAIRE :
 #define		stSubpicture	2
 #define 	stOther		3
 
+#define MAX_PGCS	100
+typedef struct {
+	int vts;
+	ifo_handle_t *	ifo;
+	vtsi_mat_t *	vtsi_mat;
+	pgcit_t *	vts_pgcit;
+	int		nrPgc;
+	pgc_t *		pgcs[MAX_PGCS];
+	unsigned long	duree;
+	int		sectors;
+	int		nrSubp;
+} Piste_t;
 
 /*@$#[dvdtools.c] global proto. AutoProtoSigV1.1. date: 106/01/18 22:49:40 */
 #include "proto.h"
@@ -42,6 +54,8 @@ char * AudioLang( int noPisteVideo, int noPisteAudio );
 char * SubLang( int noPisteVideo, int noSubtitle );
 char * AudioFormat( int noPisteVideo, int noPisteAudio );
 ssize_t MyDVDReadBlocks( dvd_file_t * dvdFile, int noSect, size_t nbSect, unsigned char * buffer );
+Piste_t * GetPiste( int noPisteVideo );
+void RazPistes();
 
 #ifdef __cplusplus
 }
