@@ -1,19 +1,3 @@
-// This file is part of Vamps.
-//
-// Vamps is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
-// (at your option) any later version.
-//
-// Vamps is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-//
-// You should have received a copy of the GNU General Public License
-// along with Vamps; if not, write to the Free Software
-// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
 // Code from libmpeg2 and mpeg2enc copyright by their respective owners
 // New code and modifications copyright Antoine Missout
 // Thanks to Sven Goethel for error resilience patches
@@ -60,6 +44,8 @@ static const double b_min_stresses[3] = { 0.00, 0.00, 0.00 };
 // so this is disabled)
 static const double max_alt_table = 0.00;
 
+#define REACT_DELAY (1024.0*128.0)
+
 // includes
 #include <stdio.h>
 #include <stdlib.h>
@@ -75,8 +61,7 @@ static const double max_alt_table = 0.00;
 // quant table
 #include "qTable.h"
 
-#include "ac.h"
-
+#include "globals.h"
 
 // useful constants
 #define I_TYPE 1
@@ -159,6 +144,7 @@ static int b_factor;
 static double i_min_stress;
 static double p_min_stress;
 static double b_min_stress;
+static int64 greedyFactor, greedyFactor2;
 
 static short quant_table_id_data[4096];
 static short *quant_table_id = &quant_table_id_data[2048];
@@ -193,10 +179,10 @@ int old_dc_pred, new_dc_pred;
 	// seq header
 	static uint horizontal_size_value;
 	static uint vertical_size_value;
-	
+
 	// pic header
 	static uint picture_coding_type;
-	
+
 	// pic code ext
 	static uint f_code[2][2];
 	static uint intra_dc_precision;
@@ -206,7 +192,7 @@ int old_dc_pred, new_dc_pred;
 	static uint q_scale_type;
 	static uint intra_vlc_format;
 	static uint alternate_scan;
-	
+
 	// error
 	static int validPicHeader;
 	static int validSeqHeader;
@@ -220,10 +206,10 @@ int old_dc_pred, new_dc_pred;
 	static int  h_offset, v_offset;
 	static int  mb_skip, mb_add;
 	static int  mb_out;
-	
+
 	static int  mb_sav_run, mb_sav_lev, mb_sav_c;
 	static short *curTable;
-	
+
 	// block data
 	typedef struct
 	{
@@ -291,7 +277,7 @@ int old_dc_pred, new_dc_pred;
 				}\
 			} else { RETURN } \
 		}
-	
+
 #else
 
 	#define WRITE \
@@ -301,7 +287,7 @@ int old_dc_pred, new_dc_pred;
 		wbuf = owbuf; \
 		mloka1 = rbuf - cbuf; if (mloka1) memmove(orbuf, cbuf, mloka1);\
 		cbuf = rbuf = orbuf; rbuf += mloka1;
-		
+
 	#define LOCK(x) \
 		while (unlikely(x > (rbuf-cbuf))) \
 		{ \
@@ -363,16 +349,16 @@ int old_dc_pred, new_dc_pred;
 		free(orbuf); \
 		free(owbuf); \
 		exit(0);
-	
+
 #endif
-	
+
 #define COPY(x)\
 		assert(x > 0); \
 		assert(wbuf + x < owbuf + BUF_SIZE); \
 		assert(cbuf + x < orbuf + BUF_SIZE); \
 		assert(cbuf + x >= orbuf); \
 		assert(wbuf + x >= orbuf); \
-		tc_memcpy(wbuf, cbuf, x);\
+		memcpy(wbuf, cbuf, x);\
 		cbuf += x; \
 		wbuf += x;
 
@@ -381,7 +367,7 @@ int old_dc_pred, new_dc_pred;
 		assert (cbuf <= rbuf); \
 		assert (cbuf < orbuf + BUF_SIZE); \
 		assert (cbuf >= orbuf);
-	
+
 #define SEEKW(x)\
 		wbuf += x; \
 		assert (wbuf < owbuf + BUF_SIZE); \
@@ -402,13 +388,13 @@ static MINLINE void putbits(uint val, int n)
 		val &= ~(0xffffffffU << n);
 		outbitcnt = BITS_IN_BUF;
 	}
-	
+
 	if (likely(n))
 	{
 		outbitbuf = (outbitbuf << n) | val;
 		outbitcnt -= n;
 	}
-	
+
 	assert(outbitcnt > 0);
 	assert(outbitcnt <= BITS_IN_BUF);
 }
@@ -445,12 +431,47 @@ static MINLINE uint Get_Bits(uint n)
 	return Val;
 }
 
+#if 1
 static MINLINE uint Copy_Bits(uint n)
 {
 	uint Val = Get_Bits(n);
 	putbits(Val, n);
 	return Val;
 }
+#else
+static MINLINE void Copy_Bits(uint n)
+{
+	uint Val = ((unsigned int)inbitbuf) >> (32 - n);
+
+	inbitcnt -= n;
+	inbitbuf <<= n;
+
+
+	while(unlikely(inbitcnt < 24)) {
+		inbitbuf |= cbuf[0] << (24 - inbitcnt);
+		inbitcnt += 8;
+		SEEKR(1)
+	}
+/*
+	if(inbitcnt < 24) {
+		inbitbuf |= cbuf[0] << (24 - inbitcnt);
+		inbitcnt += 8;
+		SEEKR(1)
+		if(inbitcnt < 24) {
+			inbitbuf |= cbuf[0] << (24 - inbitcnt);
+			inbitcnt += 8;
+			SEEKR(1)
+			if(inbitcnt < 24) {
+				inbitbuf |= cbuf[0] << (24 - inbitcnt);
+				inbitcnt += 8;
+				SEEKR(1)
+			}
+		}
+	}
+*/
+	putbits(Val, n);
+}
+#endif
 
 static MINLINE void flush_read_buffer()
 {
@@ -583,7 +604,7 @@ static int getNewQuant(int curQuant, int intra)
 			cStress = (stress_factor - i_min_stress) / (1.0 - i_min_stress);
 			mquant = intmax(scale_quant(curQuant + i_factor*cStress), increment_quant(curQuant));
 			break;
-		
+
 		case P_TYPE:
 			cStress = (stress_factor - p_min_stress) / (1.0 - p_min_stress);
 			if (intra) // since it might be used as a ref, treat it as an I frame block
@@ -591,19 +612,19 @@ static int getNewQuant(int curQuant, int intra)
 			else
 				mquant = intmax(scale_quant(curQuant + p_factor*cStress), increment_quant(curQuant));
 			break;
-			
+
 		case B_TYPE:
 			cStress = (stress_factor - b_min_stress) / (1.0 - b_min_stress);
 			mquant = intmax(scale_quant(curQuant + b_factor*cStress), increment_quant(curQuant));
 			break;
-			
+
 		default:
 			assert(0);
 			break;
 	}
-		
+
 	assert(mquant >= curQuant);
-	
+
 	return mquant;
 #endif
 }
@@ -620,9 +641,9 @@ int putAC(int run, int signed_level, int vlcformat)
 {
 	int level, len;
 	const VLCtable *ptab = NULL;
-	
+
 	level = (signed_level<0) ? -signed_level : signed_level; /* abs(signed_level) */
-	
+
 	// assert(!(run<0 || run>63 || level==0 || level>2047));
 	if(run<0 || run>63)
 	{
@@ -636,9 +657,9 @@ int putAC(int run, int signed_level, int vlcformat)
 		sliceError++;
 		return 1;
 	}
-	
+
 	len = 0;
-	
+
 	if (run<2 && level<41)
 	{
 		if (vlcformat)  ptab = &dct_code_tab1a[run][level-1];
@@ -651,7 +672,7 @@ int putAC(int run, int signed_level, int vlcformat)
 		else ptab = &dct_code_tab2[run-2][level-1];
 		len = ptab->len;
 	}
-	
+
 	if (len) /* a VLC code exists */
 	{
 		putbits(ptab->code, len);
@@ -663,7 +684,7 @@ int putAC(int run, int signed_level, int vlcformat)
 		putbits(run, 6); /* 6 bit code for run */
 		putbits(((uint)signed_level) & 0xFFF, 12);
 	}
-	
+
 	return 0;
 }
 
@@ -681,16 +702,16 @@ static MINLINE int putACfirst(int run, int val)
 void putnonintrablk(RunLevel *blk)
 {
 	assert(blk->level);
-	
+
 	if (putACfirst(blk->run, blk->level)) return;
 	blk++;
-	
+
 	while(blk->level)
 	{
 		if (putAC(blk->run, blk->level, 0)) return;
 		blk++;
 	}
-	
+
 	putbits(2,2);
 }
 
@@ -732,25 +753,25 @@ static MINLINE int get_macroblock_modes ()
     switch (picture_coding_type)
 	{
 		case I_TYPE:
-	
+
 			tab = MB_I + UBITS (bit_buf, 1);
 			DUMPBITS (bit_buf, bits, tab->len);
 			macroblock_modes = tab->modes;
-		
+
 			if ((! (frame_pred_frame_dct)) && (picture_structure == FRAME_PICTURE))
 			{
 				macroblock_modes |= UBITS (bit_buf, 1) * DCT_TYPE_INTERLACED;
 				DUMPBITS (bit_buf, bits, 1);
 			}
-		
+
 			return macroblock_modes;
-	
+
 		case P_TYPE:
-	
+
 			tab = MB_P + UBITS (bit_buf, 5);
 			DUMPBITS (bit_buf, bits, tab->len);
 			macroblock_modes = tab->modes;
-		
+
 			if (picture_structure != FRAME_PICTURE)
 			{
 				if (macroblock_modes & MACROBLOCK_MOTION_FORWARD)
@@ -780,13 +801,13 @@ static MINLINE int get_macroblock_modes ()
 				}
 				return macroblock_modes;
 			}
-	
+
 		case B_TYPE:
-	
+
 			tab = MB_B + UBITS (bit_buf, 6);
 			DUMPBITS (bit_buf, bits, tab->len);
 			macroblock_modes = tab->modes;
-		
+
 			if (picture_structure != FRAME_PICTURE)
 			{
 				if (! (macroblock_modes & MACROBLOCK_INTRA))
@@ -815,7 +836,7 @@ static MINLINE int get_macroblock_modes ()
 				}
 				return macroblock_modes;
 			}
-	
+
 		default:
 			return 0;
     }
@@ -827,15 +848,15 @@ static MINLINE int get_quantizer_scale ()
     int quantizer_scale_code;
 
     quantizer_scale_code = UBITS (bit_buf, 5);
-	DUMPBITS (bit_buf, bits, 5); 
-	
+	DUMPBITS (bit_buf, bits, 5);
+
 	if (!quantizer_scale_code)
     {
 		DEBF("illegal quant scale code: %d\n", quantizer_scale_code);
 		sliceError++;
 		quantizer_scale_code++;
     }
-	
+
 	if (q_scale_type) return non_linear_quantizer_scale[quantizer_scale_code];
     else return quantizer_scale_code << 1;
 }
@@ -984,7 +1005,7 @@ static void get_intra_block_B14 ()
     int i, li;
     int val;
     const DCTtab * tab;
-	
+
     li = i = 0;
 
     while (1)
@@ -992,10 +1013,10 @@ static void get_intra_block_B14 ()
 		if (bit_buf >= 0x28000000)
 		{
 			tab = DCT_B14AC_5 + (UBITS (bit_buf, 5) - 5);
-	
+
 			i += tab->run;
 			if (i >= 64) break;	/* end of block */
-	
+
 	normal_code:
 			DUMPBITS (bit_buf, bits, tab->len);
 			val = tab->level;
@@ -1006,26 +1027,26 @@ static void get_intra_block_B14 ()
 				if (putAC(i - li - 1, val, 0)) break;
 				li = i;
 			}
-	
+
 			DUMPBITS (bit_buf, bits, 1);
-	
+
 			continue;
 		}
 		else if (bit_buf >= 0x04000000)
 		{
 			tab = DCT_B14_8 + (UBITS (bit_buf, 8) - 4);
-	
+
 			i += tab->run;
 			if (i < 64) goto normal_code;
-	
+
 			/* escape code */
 			i += (UBITS (bit_buf, 12) & 0x3F) - 64;
-			if (i >= 64) 
+			if (i >= 64)
 			{
 				sliceError++;
 				break;	/* illegal, check needed to avoid buffer overflow */
 			}
-			
+
 			DUMPBITS (bit_buf, bits, 12);
 			val = SBITS (bit_buf, 12);
 			UPDATE_VAL
@@ -1034,9 +1055,9 @@ static void get_intra_block_B14 ()
 				if (putAC(i - li - 1, val, 0)) break;
 				li = i;
 			}
-	
+
 			DUMPBITS (bit_buf, bits, 12);
-	
+
 			continue;
 		}
 		else if (bit_buf >= 0x02000000)
@@ -1067,7 +1088,7 @@ static void get_intra_block_B14 ()
 		sliceError++;
 		break;	/* illegal, check needed to avoid buffer overflow */
 	}
-	
+
 	COPYBITS (bit_buf, bits, 2);	/* end of block code */
 }
 
@@ -1077,7 +1098,7 @@ static void get_intra_block_B15 ()
     int i, li;
     int val;
     const DCTtab * tab;
-	
+
     li = i = 0;
 
     while (1)
@@ -1085,13 +1106,13 @@ static void get_intra_block_B15 ()
 		if (bit_buf >= 0x04000000)
 		{
 			tab = DCT_B15_8 + (UBITS (bit_buf, 8) - 4);
-	
+
 			i += tab->run;
 			if (i < 64)
 			{
 	normal_code:
 				DUMPBITS (bit_buf, bits, tab->len);
-				
+
 				val = tab->level;
 				val = (val ^ SBITS (bit_buf, 1)) - SBITS (bit_buf, 1);
 				UPDATE_VAL
@@ -1100,23 +1121,23 @@ static void get_intra_block_B15 ()
 					if (putAC(i - li - 1, val, 1)) break;
 					li = i;
 				}
-		
+
 				DUMPBITS (bit_buf, bits, 1);
-		
+
 				continue;
 			}
 			else
 			{
 				if (i >= 128) break; /* end of block */
-			
+
 				i += (UBITS (bit_buf, 12) & 0x3F) - 64;
-				
+
 				if (i >= 64)
 				{
 					sliceError++;
 					break;	/* illegal, check against buffer overflow */
 				}
-				
+
 				DUMPBITS (bit_buf, bits, 12);
 				val = SBITS (bit_buf, 12);
 				UPDATE_VAL
@@ -1125,9 +1146,9 @@ static void get_intra_block_B15 ()
 					if (putAC(i - li - 1, val, 1)) break;
 					li = i;
 				}
-		
+
 				DUMPBITS (bit_buf, bits, 12);
-		
+
 				continue;
 			}
 		}
@@ -1186,14 +1207,14 @@ static int get_non_intra_block_rq (RunLevel *blk)
 		if (bit_buf >= 0x28000000)
 		{
 			tab = DCT_B14AC_5 + (UBITS (bit_buf, 5) - 5);
-	
+
 	entry_1:
 			i += tab->run;
 			if (i >= 64)
 			break;	/* end of block */
-	
+
 	normal_code:
-	
+
 			DUMPBITS (bit_buf, bits, tab->len);
 			val = tab->level;
 			val = (val ^ SBITS (bit_buf, 1)) - SBITS (bit_buf, 1);
@@ -1205,30 +1226,30 @@ static int get_non_intra_block_rq (RunLevel *blk)
 				li = i;
 				blk++;
 			}
-	
+
 			DUMPBITS (bit_buf, bits, 1);
-	
+
 			continue;
 		}
-	
+
 	entry_2:
 		if (bit_buf >= 0x04000000)
 		{
 			tab = DCT_B14_8 + (UBITS (bit_buf, 8) - 4);
-	
+
 			i += tab->run;
 			if (i < 64) goto normal_code;
-	
+
 			/* escape code */
-	
+
 			i += (UBITS (bit_buf, 12) & 0x3F) - 64;
-			
+
 			if (i >= 64)
 			{
 				sliceError++;
 				break;	/* illegal, check needed to avoid buffer overflow */
 			}
-			
+
 			DUMPBITS (bit_buf, bits, 12);
 			val = SBITS (bit_buf, 12);
 			UPDATE_VAL
@@ -1239,9 +1260,9 @@ static int get_non_intra_block_rq (RunLevel *blk)
 				li = i;
 				blk++;
 			}
-			
+
 			DUMPBITS (bit_buf, bits, 12);
-	
+
 			continue;
 		}
 		else if (bit_buf >= 0x02000000)
@@ -1266,16 +1287,16 @@ static int get_non_intra_block_rq (RunLevel *blk)
 		{
 			tab = DCT_16 + UBITS (bit_buf, 16);
 			DUMPBITS (bit_buf, bits, 16);
-			
+
 			i += tab->run;
 			if (i < 64) goto normal_code;
 		}
-		
+
 		sliceError++;
 		break;	/* illegal, check needed to avoid buffer overflow */
 	}
     DUMPBITS (bit_buf, bits, 2);	/* dump end of block code */
-	
+
 	blk->level = 0;
 
     return i;
@@ -1320,7 +1341,7 @@ static int get_non_intra_block_sav (RunLevel *blk, int cc)
     int i, li;
     int val;
     const DCTtab * tab;
-	
+
 #ifdef P_FRAME_NON_INTRA_DROP
 	#if (P_FRAME_NON_INTRA_DROP < 0)
 		RunLevel *oblk = blk;
@@ -1343,14 +1364,14 @@ static int get_non_intra_block_sav (RunLevel *blk, int cc)
 		if (bit_buf >= 0x28000000)
 		{
 			tab = DCT_B14AC_5 + (UBITS (bit_buf, 5) - 5);
-	
+
 	entry_1:
 			i += tab->run;
 			if (i >= 64)
 			break;	/* end of block */
-	
+
 	normal_code:
-	
+
 			DUMPBITS (bit_buf, bits, tab->len);
 			val = tab->level;
 			val = (val ^ SBITS (bit_buf, 1)) - SBITS (bit_buf, 1);
@@ -1368,28 +1389,28 @@ static int get_non_intra_block_sav (RunLevel *blk, int cc)
 			WRITE_VAL
 
 			DUMPBITS (bit_buf, bits, 1);
-	
+
 			continue;
 		}
-	
+
 	entry_2:
 		if (bit_buf >= 0x04000000)
 		{
 			tab = DCT_B14_8 + (UBITS (bit_buf, 8) - 4);
-	
+
 			i += tab->run;
 			if (i < 64) goto normal_code;
-	
+
 			/* escape code */
-	
+
 			i += (UBITS (bit_buf, 12) & 0x3F) - 64;
-			
+
 			if (i >= 64)
 			{
 				sliceError++;
 				break;	/* illegal, check needed to avoid buffer overflow */
 			}
-			
+
 			DUMPBITS (bit_buf, bits, 12);
 			val = SBITS (bit_buf, 12);
 			SAVE_VAL
@@ -1404,9 +1425,9 @@ static int get_non_intra_block_sav (RunLevel *blk, int cc)
 			}
 			UPDATE_VAL
 			WRITE_VAL
-			
+
 			DUMPBITS (bit_buf, bits, 12);
-	
+
 			continue;
 		}
 		else if (bit_buf >= 0x02000000)
@@ -1431,16 +1452,16 @@ static int get_non_intra_block_sav (RunLevel *blk, int cc)
 		{
 			tab = DCT_16 + UBITS (bit_buf, 16);
 			DUMPBITS (bit_buf, bits, 16);
-			
+
 			i += tab->run;
 			if (i < 64) goto normal_code;
 		}
-		
+
 		sliceError++;
 		break;	/* illegal, check needed to avoid buffer overflow */
 	}
     DUMPBITS (bit_buf, bits, 2);	/* dump end of block code */
-	
+
 #ifdef P_FRAME_NON_INTRA_DROP
 	#if (P_FRAME_NON_INTRA_DROP < 0)
 		blk -= (int)((blk - oblk) * (stress_factor / P_FRAME_NON_INTRA_DROP));
@@ -1451,7 +1472,7 @@ static int get_non_intra_block_sav (RunLevel *blk, int cc)
 #endif
 
 	blk->level = 0;
-	
+
     return i;
 }
 
@@ -1482,14 +1503,14 @@ static int get_non_intra_block_drop (RunLevel *blk, int cc)
 		if (bit_buf >= 0x28000000)
 		{
 			tab = DCT_B14AC_5 + (UBITS (bit_buf, 5) - 5);
-	
+
 	entry_1:
 			i += tab->run;
 			if (i >= 64)
 			break;	/* end of block */
-	
+
 	normal_code:
-	
+
 			DUMPBITS (bit_buf, bits, tab->len);
 			val = tab->level;
 			val = (val ^ SBITS (bit_buf, 1)) - SBITS (bit_buf, 1);
@@ -1498,36 +1519,36 @@ static int get_non_intra_block_drop (RunLevel *blk, int cc)
 			WRITE_VAL
 
 			DUMPBITS (bit_buf, bits, 1);
-	
+
 			continue;
 		}
-	
+
 	entry_2:
 		if (bit_buf >= 0x04000000)
 		{
 			tab = DCT_B14_8 + (UBITS (bit_buf, 8) - 4);
-	
+
 			i += tab->run;
 			if (i < 64) goto normal_code;
-	
+
 			/* escape code */
-	
+
 			i += (UBITS (bit_buf, 12) & 0x3F) - 64;
-			
+
 			if (i >= 64)
 			{
 				sliceError++;
 				break;	/* illegal, check needed to avoid buffer overflow */
 			}
-			
+
 			DUMPBITS (bit_buf, bits, 12);
 			val = SBITS (bit_buf, 12);
 			SAVE_VAL
 			UPDATE_VAL
 			WRITE_VAL
-			
+
 			DUMPBITS (bit_buf, bits, 12);
-	
+
 			continue;
 		}
 		else if (bit_buf >= 0x02000000)
@@ -1552,23 +1573,23 @@ static int get_non_intra_block_drop (RunLevel *blk, int cc)
 		{
 			tab = DCT_16 + UBITS (bit_buf, 16);
 			DUMPBITS (bit_buf, bits, 16);
-			
+
 			i += tab->run;
 			if (i < 64) goto normal_code;
 		}
-		
+
 		sliceError++;
 		break;	/* illegal, check needed to avoid buffer overflow */
 	}
     DUMPBITS (bit_buf, bits, 2);	/* dump end of block code */
-	
+
 	#if (P_FRAME_NON_INTRA_DROP < 0)
 		blk -= (int)((blk - oblk) * (stress_factor / P_FRAME_NON_INTRA_DROP));
 		#ifdef DEMO
 			if ((gopCount & 0x7F) < 10) blk = oblk;
 		#endif
 	#endif
-	
+
 	blk->level = 0;
 
     return i;
@@ -1606,25 +1627,25 @@ static MINLINE void slice_intra_DCT (const int cc)
 		{
 			val = bri + (128 << intra_dc_precision);
 			old_dc_pred = val;
-				
+
 				val += delta_bright << intra_dc_precision;
 				if (val > (255 << intra_dc_precision)) val = 255 << intra_dc_precision;
 				else if (val < 0) val = 0;
-					
+
 			bri = val - (128 << intra_dc_precision);
 			new_dc_pred = val;
-			
+
 			dc_reset = 0;
 		}
 		else
 		{
 			val = bri + old_dc_pred;
 			old_dc_pred = val;
-			
+
 				val += delta_bright << intra_dc_precision;
 				if (val > (255 << intra_dc_precision)) val = 255 << intra_dc_precision;
 				else if (val < 0) val = 0;
-				
+
 			bri = val - new_dc_pred;
 			new_dc_pred = val;
 		}
@@ -1755,14 +1776,14 @@ do {															\
 void putmbdata(int macroblock_modes)
 {
 		putmbtype(macroblock_modes & 0x1F);
-		
+
 		/*switch (picture_coding_type)
 		{
 			case I_TYPE:
 				if ((! (frame_pred_frame_dct)) && (picture_structure == FRAME_PICTURE))
 					putbits(macroblock_modes & DCT_TYPE_INTERLACED ? 1 : 0, 1);
 				break;
-		
+
 			case P_TYPE:
 				if (picture_structure != FRAME_PICTURE)
 				{
@@ -1779,7 +1800,7 @@ void putmbdata(int macroblock_modes)
 						putbits(macroblock_modes & DCT_TYPE_INTERLACED ? 1 : 0, 1);
 					break;
 				}
-		
+
 			case B_TYPE:
 				if (picture_structure != FRAME_PICTURE)
 				{
@@ -1858,7 +1879,7 @@ static MINLINE int slice_init (int code)
     quantizer_scale = get_quantizer_scale ();
 	new_quantizer_scale = getNewQuant(quantizer_scale, 0);
 	put_quantiser(new_quantizer_scale);
-	
+
 
     /* ignore intra_slice and all the extra data */
     while (bit_buf & 0x80000000)
@@ -1895,7 +1916,7 @@ static MINLINE int slice_init (int code)
 	mb_skip = 0;
 	COPYBITS (bit_buf, bits, 1);
 	DUMPBITS(bit_buf, bits, mba->len);
-	
+
     h_offset = (offset + mba->mba) << 4;
 
     while (h_offset - (int)horizontal_size_value >= 0)
@@ -1925,7 +1946,7 @@ void mpeg2_slice ( const int code )
 		int macroblock_modes;
 		int mba_inc;
 		const MBAtab * mba;
-	
+
 		macroblock_modes = get_macroblock_modes ();
 		if (macroblock_modes & MACROBLOCK_QUANT) quantizer_scale = get_quantizer_scale ();
 
@@ -1943,17 +1964,17 @@ void mpeg2_slice ( const int code )
 			putaddrinc(mb_add + mb_skip); mb_skip = 0;
 			putmbdata(macroblock_modes);
 			if (macroblock_modes & MACROBLOCK_QUANT) put_quantiser(new_quantizer_scale);
-		
+
 			if (concealment_motion_vectors)
 			{
 				if (picture_structure == FRAME_PICTURE) motion_fr_conceal ();
 				else motion_fi_conceal ();
 			}
-			
+
 			curTable = quant_tables[quant_equ[quantizer_scale]][quant_equ[new_quantizer_scale]];
 			if (!curTable)
 			{
-				/*DEBF("Inv. curTable: qs: %i nqs: %i qe_qs: %i qe_nqs: %i\n", 
+				/*DEBF("Inv. curTable: qs: %i nqs: %i qe_qs: %i qe_nqs: %i\n",
 						quantizer_scale, new_quantizer_scale,
 						quant_equ[quantizer_scale], quant_equ[new_quantizer_scale]);*/
 				curTable = quant_table_id;
@@ -1974,7 +1995,7 @@ void mpeg2_slice ( const int code )
 			int batb;
 			uint8	n_owbuf[32], *n_wbuf, *o_owbuf, *o_wbuf;
 			uint32	n_outbitcnt, n_outbitbuf, o_outbitcnt, o_outbitbuf;
-			
+
 #ifdef CHANGE_BRIGHTNESS
 			dc_reset = 1;
 #endif
@@ -1984,13 +2005,13 @@ void mpeg2_slice ( const int code )
 			o_outbitcnt = outbitcnt; o_outbitbuf = outbitbuf; \
 			owbuf = wbuf = n_owbuf; \
 			outbitcnt = BITS_IN_BUF; outbitbuf = 0;
-			
+
 #define POP_BIT_IO \
 			n_wbuf = wbuf; \
 			n_outbitcnt = outbitcnt; n_outbitbuf = outbitbuf; \
 			owbuf = o_owbuf; wbuf = o_wbuf; \
 			outbitcnt = o_outbitcnt; outbitbuf = o_outbitbuf;
-			
+
 			PUSH_BIT_IO
 
 			if (picture_structure == FRAME_PICTURE)
@@ -2007,11 +2028,11 @@ void mpeg2_slice ( const int code )
 					case MC_16X8: MOTION_CALL (motion_fi_16x8, macroblock_modes); break;
 					case MC_DMV: MOTION_CALL (motion_fi_dmv, MACROBLOCK_MOTION_FORWARD); break;
 				}
-				
+
 			POP_BIT_IO
-			
+
 			// end saving data
-			
+
 #ifdef STAT
 			if (picture_coding_type == P_TYPE) cnt_p_ni++;
 			else if (picture_coding_type == B_TYPE) cnt_b_ni++;
@@ -2021,12 +2042,12 @@ void mpeg2_slice ( const int code )
 			if (macroblock_modes & MACROBLOCK_PATTERN)
 			{
 				int coded_block_pattern = get_coded_block_pattern ();
-				
+
 				mb_sav_lev = 0xFFFF;
 				curTable = quant_tables[quant_equ[quantizer_scale]][quant_equ[new_quantizer_scale]];
 				if (!curTable)
 				{
-					/*DEBF("Inv. curTable: qs: %i nqs: %i qe_qs: %i qe_nqs: %i\n", 
+					/*DEBF("Inv. curTable: qs: %i nqs: %i qe_qs: %i qe_nqs: %i\n",
 							quantizer_scale, new_quantizer_scale,
 							quant_equ[quantizer_scale], quant_equ[new_quantizer_scale]);*/
 					curTable = quant_table_id;
@@ -2049,15 +2070,15 @@ void mpeg2_slice ( const int code )
 					{
 						assert(n_wbuf == n_owbuf);
 						assert(n_outbitcnt == BITS_IN_BUF);
-					
+
 						if ((h_offset == 0) || (h_offset == horizontal_size_value - 16))  // can't skip last mb
 						{
 							// we can't transmit mv (0,0) since PMV could be different than 0 for last block
 							// so we transmit the single smallest coeff. instead unrequantised
 							// anyway this is likely to take no more bit than transmiting a null mv....
-							
+
 							assert((mb_sav_lev) && (mb_sav_lev != 0xFFFF));
-							
+
 							new_coded_block_pattern = 1 << (5 - mb_sav_c);
 							macroblock_modes |= MACROBLOCK_PATTERN;
 							new_quantizer_scale = quantizer_scale;
@@ -2085,11 +2106,11 @@ void mpeg2_slice ( const int code )
 			for (batb = 0; batb < (n_wbuf - n_owbuf); batb++) putbits(n_owbuf[batb], 8);
 			putbits(n_outbitbuf, BITS_IN_BUF - n_outbitcnt);
 			// end saved motion data...
-			
+
 			if (macroblock_modes & MACROBLOCK_PATTERN)
 			{
 				putcbp(new_coded_block_pattern);
-				
+
 				if (new_coded_block_pattern & 0x20) putnonintrablk(block[0]);
 				if (new_coded_block_pattern & 0x10) putnonintrablk(block[1]);
 				if (new_coded_block_pattern & 0x08) putnonintrablk(block[2]);
@@ -2102,7 +2123,7 @@ void mpeg2_slice ( const int code )
 skip_mb:
 
 		NEXT_MACROBLOCK;
-	
+
 		mba_inc = 0;
 		while (1)
 		{
@@ -2131,14 +2152,14 @@ skip_mb:
 				}
 		}
 		DUMPBITS (bit_buf, bits, mba->len); //PPP
-		
+
 		mba_inc += mba->mba;
 		mb_add = mba_inc + 1;
-		
+
 #ifdef CHANGE_BRIGHTNESS
 		if (mba_inc) dc_reset = 1;
 #endif
-	
+
 		if (mba_inc) do { NEXT_MACROBLOCK; } while (--mba_inc);
     }
 
@@ -2176,7 +2197,7 @@ skip_mb:
 	#endif
 #endif
 
-int quantisers[42] = 
+int quantisers[42] =
 {
 	1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 32,
 	34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 54, 56, 58, 60, 62, 64, 72, 80,
@@ -2190,9 +2211,8 @@ int main (int argc, const char * argv[])
 #endif
 {
 	uint8 ID, found;
-	int64 greedyFactor, greedyFactor2;
 	int i;
-	
+
 #ifdef DEMO
 	gopCount = 0;
 #endif
@@ -2232,11 +2252,11 @@ int main (int argc, const char * argv[])
 	inbytecnt = outbytecnt = 0;
 	eof = 0;
 #endif
-	
+
 	validPicHeader = 0;
 	validSeqHeader = 0;
 	validExtHeader = 0;
-	
+
 #ifndef THREAD
 	// argument parsing
 #ifdef CHANGE_BRIGHTNESS
@@ -2249,16 +2269,22 @@ int main (int argc, const char * argv[])
 	sscanf(argv[3], "%lld", &orim2vsize);
 #endif
 
+/* added */
+#ifdef THREAD
+	orim2vsize = rqt_visize;
+	fact_x     = rqt_fact;
+#endif
+
 	greedyFactor = orim2vsize / 100;
 	greedyFactor2 = orim2vsize / 50;
-	
+
 #ifndef THREAD
 	if (fact_x <= 1.0)
 	{
 		unsigned char buf[4096];
-		
+
 		while(1)
-		{		
+		{
 			int i = read(0, buf, 4096);
 			if (i > 0) write(1, buf, i);
 			else return 0;
@@ -2266,7 +2292,7 @@ int main (int argc, const char * argv[])
 	}
 	else if (fact_x > 900.0) fact_x = 900.0;
 #endif
-	
+
 	// factor and stresses setting
 	if (fact_x <= 1.0)
 	{
@@ -2306,62 +2332,62 @@ int main (int argc, const char * argv[])
 		p_min_stress = p_min_stresses[1] + inter * (p_min_stresses[2] - p_min_stresses[1]);
 		b_min_stress = b_min_stresses[1] + inter * (b_min_stresses[2] - b_min_stresses[1]);
 	}
-	
+
 	/*LOGF(   "i_factor: %i p_factor: %i b_factor: %i\n"
 			"i_min_stress: %.02f p_min_stress: %.02f b_min_stress: %.02f\n",
 			i_factor, p_factor, b_factor,
 			i_min_stress, p_min_stress, b_min_stress);*/
-	
+
 	// fill quant table
 	// id table
 	for (i = -2048; i <= 2047; i++) quant_table_id[i] = i;
-	
+
 	// other tables
 	for (i = 0; i < 42; i++)
 	{
 		int q = quantisers[i];
 		int j;
-		
+
 		for (j = i + 1; j < 42; j++)
 		{
 			int nq = quantisers[j];
 			int k;
 			short *cTab = quant_tables[quant_equ[q]][quant_equ[nq]];
-			
+
 			for (k = -2048; k <= 2047; k++)
 			{
 				int ov = k*q;
 				int t = ov / nq;
-			
+
 				if (fact_x <= max_alt_table)
 				{
 					int t2, t3;
 					int d, d2, d3;
 					int nv, nv2, nv3;
-					
+
 					t2 = t + 1;
 					t3 = t - 1;
-					
+
 					nv = t * nq;
 					nv2 = t2 * nq;
 					nv3 = t3 * nq;
-					
+
 					d = abs(nv - ov);
 					d2 = abs(nv2 - ov);
 					d3 = abs(nv3 - ov);
-					
+
 					if (d2 < d) { d = d2; t = t2; }
 					if (d3 < d) t = t3;
 				}
-	
+
 				if (t > 2047) t = 2047;
 				else if (t < -2048) t = -2048;
-	
+
 				cTab[k] = t;
 			}
 		}
 	}
-	
+
 #ifndef THREAD
 	LOG("M2VRequantiser by Makira.\n");
 #ifdef WIN
@@ -2384,13 +2410,13 @@ int main (int argc, const char * argv[])
 			LOCK(8)
 			if		(		(cbuf[7] == 0) && (cbuf[6] == 0) && (cbuf[5] == 0) && (cbuf[4] == 0)
 						&&  (cbuf[3] == 0) && (cbuf[2] == 0) && (cbuf[1] == 0) && (cbuf[0] == 0) ) { SEEKR(1) }
-			else 
+			else
 #endif
 			if ( (cbuf[0] == 0) && (cbuf[1] == 0) && (cbuf[2] == 1) ) found = 1; // start code !
 			else { COPY(1) } // continue search
 		}
 		COPY(3)
-		
+
 		// get start code
 		LOCK(1)
 		ID = cbuf[0];
@@ -2410,9 +2436,9 @@ int main (int argc, const char * argv[])
 				validPicHeader = 1;
 				cbuf[1] |= 0x7; cbuf[2] = 0xFF; cbuf[3] |= 0xF8; // vbv_delay is now 0xFFFF
 			}
-			
+
 			validExtHeader = 0;
-			
+
 			COPY(4)
 		}
 		else if (ID == 0xB3) // seq header
@@ -2429,10 +2455,10 @@ int main (int argc, const char * argv[])
 			}
 			else
 				validSeqHeader = 1;
-				
+
 			validPicHeader = 0;
 			validExtHeader = 0;
-			
+
 			COPY(8)
 		}
 		else if (ID == 0xB5) // extension
@@ -2446,7 +2472,7 @@ int main (int argc, const char * argv[])
 				f_code[0][1] = (cbuf[1] >> 4) - 1;
 				f_code[1][0] = (cbuf[1] & 0xF) - 1;
 				f_code[1][1] = (cbuf[2] >> 4) - 1;
-				
+
 				intra_dc_precision = (cbuf[2] >> 2) & 0x3;
 				picture_structure = cbuf[2] & 0x3;
 				frame_pred_frame_dct = (cbuf[3] >> 6) & 0x1;
@@ -2454,7 +2480,7 @@ int main (int argc, const char * argv[])
 				q_scale_type = (cbuf[3] >> 4) & 0x1;
 				intra_vlc_format = (cbuf[3] >> 3) & 0x1;
 				alternate_scan = (cbuf[3] >> 2) & 0x1;
-				
+
 				if (	(f_code[0][0] > 8 && f_code[0][0] < 14)
 					||  (f_code[0][1] > 8 && f_code[0][1] < 14)
 					||  (f_code[1][0] > 8 && f_code[1][0] < 14)
@@ -2487,23 +2513,27 @@ int main (int argc, const char * argv[])
 		{
 			uint8 *outTemp = wbuf, *inTemp = cbuf;
 			int64 threshold;
-			
+
 			bytediff = (outbytecnt + (wbuf - owbuf)) - ((inbytecnt - (rbuf - cbuf)) / fact_x);
-			
+
 
 			if (inbytecnt < greedyFactor2) threshold = inbytecnt >> 1;
 			else if (orim2vsize - inbytecnt < greedyFactor2) threshold = (orim2vsize - inbytecnt) >> 1;
 			else threshold = greedyFactor;
-			
+
 			if (threshold < 1024) threshold = 1024;
-			
+
 			stress_factor = (float)(bytediff + threshold) / (float)(threshold << 1);
 			if (stress_factor > 1.0f)		stress_factor = 1.0f;
 			else if (stress_factor < 0.0f)	stress_factor = 0.0f;
 
+#if 0
+			stress_factor = (((inbytecnt - (rbuf - cbuf)) / fact_x) - (outbytecnt + (wbuf - owbuf))) / REACT_DELAY;
+#endif
+
 
 #ifdef LOG_RATE_CONTROL
-			/*fprintf(LOG_FILE, "%f%%: Requested: %f  Current: %f  Delta: %lld   Threshold: %f  Stress: %f\n", 
+			/*fprintf(LOG_FILE, "%f%%: Requested: %f  Current: %f  Delta: %lld   Threshold: %f  Stress: %f\n",
 				(float)(100.0f*inbytecnt)/orim2vsize,		//	percent
 				(float)fact_x,								//	requested
 				(float)(inbytecnt - (rbuf - cbuf))/(float)(outbytecnt + (wbuf - owbuf)), //	current
@@ -2514,7 +2544,7 @@ int main (int argc, const char * argv[])
 			fprintf(LOG_FILE, "inb: %.0f inb_c: %.0f oub: %.0f oub_c: %.0f cur: %.3f dif: %.0f thr: %.0f str: %.03f\n",
 				(float)inbytecnt,
 				(float)(inbytecnt - (rbuf - cbuf)),
-				(float)outbytecnt, 
+				(float)outbytecnt,
 				(float)(outbytecnt + (wbuf - owbuf)),
 				(float)(inbytecnt - (rbuf - cbuf))/(float)(outbytecnt + (wbuf - owbuf)),
 				(float)bytediff,
@@ -2522,7 +2552,12 @@ int main (int argc, const char * argv[])
 				(float)stress_factor );
 #endif
 
-
+#if 0
+			if 	(		((picture_coding_type == B_TYPE) && (stress_factor < 2.5f)) // don't recompress if we're in advance!
+					||	((picture_coding_type == P_TYPE) && (stress_factor < -2.5f))
+					||	((picture_coding_type == I_TYPE) && (stress_factor < -5.0f))
+				)
+#else
 #ifndef CHANGE_BRIGHTNESS
 			if (	((picture_coding_type == I_TYPE) && ( stress_factor > i_min_stress))
 				||  ((picture_coding_type == P_TYPE) && ( stress_factor > p_min_stress))
@@ -2531,6 +2566,7 @@ int main (int argc, const char * argv[])
 				||	((gopCount & 0x7F) < 10)
 #endif
 				)
+#endif
 #endif
 			{
 
@@ -2540,20 +2576,20 @@ int main (int argc, const char * argv[])
 				{
 					// try to read more data
 					if (!eof)
-					{ 
-						mloka2 = MIN_READ; 
-						while (mloka2) 
+					{
+						mloka2 = MIN_READ;
+						while (mloka2)
 						{
-							assert(rbuf + mloka2 < orbuf + BUF_SIZE); 
+							assert(rbuf + mloka2 < orbuf + BUF_SIZE);
 #ifdef USE_FD
 							mloka1 = fread(rbuf, 1, mloka2, ifd);
 #else
 							mloka1 = read(0, rbuf, mloka2);
-#endif							
-							if (mloka1 <= 0) { eof = 1; break; } 
-							inbytecnt += mloka1; 
-							rbuf += mloka1; 
-							mloka2 -= mloka1; 
+#endif
+							if (mloka1 <= 0) { eof = 1; break; }
+							inbytecnt += mloka1;
+							rbuf += mloka1;
+							mloka2 -= mloka1;
 						}
 					}
 					// lock slice manually
@@ -2565,33 +2601,33 @@ int main (int argc, const char * argv[])
 						{
 							toLock = nsc - cbuf + 3;
 							LOCK(toLock)
-			
+
 							if ( (nsc[0] == 0) && (nsc[1] == 0) && (nsc[2] == 1) ) fsc = 1; // start code !
 							else nsc++; // continue search
 						}
 					}
 				}
 #endif
-				
+
 				// init error
 				sliceError = 0;
-			
+
 				// init bit buffer
 				inbitbuf = 0; inbitcnt = 0;
 				outbitbuf = 0; outbitcnt = BITS_IN_BUF;
-				
+
 				// get 32 bits
 				Refill_bits();
 				Refill_bits();
 				Refill_bits();
 				Refill_bits();
-			
+
 				// begin bit level recoding
 				mpeg2_slice(ID);
 				flush_read_buffer();
 				flush_write_buffer();
 				// end bit level recoding
-			
+
 #ifndef CHANGE_BRIGHTNESS
 				if ((wbuf - outTemp > cbuf - inTemp) || (sliceError > MAX_ERRORS)) // yes that might happen, rarely
 				{
@@ -2603,14 +2639,14 @@ int main (int argc, const char * argv[])
 #endif
 
 					// in this case, we'll just use the original slice !
-					tc_memcpy(outTemp, inTemp, cbuf - inTemp);
+					memcpy(outTemp, inTemp, cbuf - inTemp);
 					wbuf = outTemp + (cbuf - inTemp);
-					
+
 					// adjust outbytecnt
 					outbytecnt -= (wbuf - outTemp) - (cbuf - inTemp);
 				}
 #endif
-				
+
 #ifdef STAT
 	#ifdef LOG_RATE_CONTROL
 			if (picture_coding_type == I_TYPE) fprintf(LOG_FILE, "-I-\n");
@@ -2622,19 +2658,19 @@ int main (int argc, const char * argv[])
 						new_i += (wbuf - outTemp > cbuf - inTemp) ? (cbuf - inTemp) : (wbuf - outTemp);
 						cnt_i ++;
 						break;
-						
+
 					case P_TYPE:
 						ori_p += cbuf - inTemp;
 						new_p += (wbuf - outTemp > cbuf - inTemp) ? (cbuf - inTemp) : (wbuf - outTemp);
 						cnt_p ++;
 						break;
-						
+
 					case B_TYPE:
 						ori_b += cbuf - inTemp;
 						new_b += (wbuf - outTemp > cbuf - inTemp) ? (cbuf - inTemp) : (wbuf - outTemp);
 						cnt_b ++;
 						break;
-						
+
 					default:
 						assert(0);
 						break;
@@ -2642,7 +2678,7 @@ int main (int argc, const char * argv[])
 #endif
 			}
 		}
-		
+
 #ifndef NDEBUG
 		if ((ID >= 0x01) && (ID <= 0xAF) && (!validPicHeader || !validSeqHeader || !validExtHeader))
 		{
@@ -2661,7 +2697,6 @@ int main (int argc, const char * argv[])
 	// keeps gcc happy
 	return 0;
 }
-
 /*------------------------------------------------------------------------------
 	REQUANT_INITVAR-
 Linux!jef 2006/02/06 23:14:17
@@ -2719,3 +2754,61 @@ void requant_initvar()
 	curTable = NULL;
 	memset( block, 0, sizeof(block));
 }
+/*------------------------------------------------------------------------------
+	REQUANT_SETFACTOR-
+Linux!jef 2006/11/09 21:58:44
+------------------------------------------------------------------------------*/
+
+void requant_factorchange()
+{
+	orim2vsize = rqt_visize;
+
+	greedyFactor = orim2vsize / 100;
+	greedyFactor2 = orim2vsize / 50;
+
+	if( fact_x == rqt_fact )	return;
+
+	fact_x     = rqt_fact;
+	DBG('r',fprintf(stderr,"%s: fact_x=%f orim2vsize=%lld\n", __FUNCTION__, fact_x, orim2vsize ););
+
+	// factor and stresses setting
+	if (fact_x <= 1.0)
+	{
+		i_factor = i_factors[0];
+		p_factor = p_factors[0];
+		b_factor = b_factors[0];
+		i_min_stress = i_min_stresses[0];
+		p_min_stress = p_min_stresses[0];
+		b_min_stress = b_min_stresses[0];
+	}
+	else if (fact_x >= 10.0)
+	{
+		i_factor = i_factors[2];
+		p_factor = p_factors[2];
+		b_factor = b_factors[2];
+		i_min_stress = i_min_stresses[2];
+		p_min_stress = p_min_stresses[2];
+		b_min_stress = b_min_stresses[2];
+	}
+	else if (fact_x <= 3.0) // 1.0 .. 3.0
+	{
+		double inter = (fact_x - 1.0)/(3.0 - 1.0);
+		i_factor = i_factors[0] + inter * (i_factors[1] - i_factors[0]);
+		p_factor = p_factors[0] + inter * (p_factors[1] - p_factors[0]);
+		b_factor = b_factors[0] + inter * (b_factors[1] - b_factors[0]);
+		i_min_stress = i_min_stresses[0] + inter * (i_min_stresses[1] - i_min_stresses[0]);
+		p_min_stress = p_min_stresses[0] + inter * (p_min_stresses[1] - p_min_stresses[0]);
+		b_min_stress = b_min_stresses[0] + inter * (b_min_stresses[1] - b_min_stresses[0]);
+	}
+	else // 3.0 .. 10.0
+	{
+		double inter = (fact_x - 3.0)/(10.0 - 3.0);
+		i_factor = i_factors[1] + inter * (i_factors[2] - i_factors[1]);
+		p_factor = p_factors[1] + inter * (p_factors[2] - p_factors[1]);
+		b_factor = b_factors[1] + inter * (b_factors[2] - b_factors[1]);
+		i_min_stress = i_min_stresses[1] + inter * (i_min_stresses[2] - i_min_stresses[1]);
+		p_min_stress = p_min_stresses[1] + inter * (p_min_stresses[2] - p_min_stresses[1]);
+		b_min_stress = b_min_stresses[1] + inter * (b_min_stresses[2] - b_min_stresses[1]);
+	}
+}
+
