@@ -28,7 +28,13 @@ long long FsFree PROTO((char *fsName));
 int DirExist PROTO((char *dirName));
 int FileExist PROTO((char *fileName));
 FILE *my_popen PROTO((char *command, char *mode));
-int my_pclose PROTO((register FILE *stream));
+int my_pclose PROTO((FILE *stream));
+char *Duree2String PROTO((int duree));
+char *AddString2List PROTO((char *list, char *string));
+char *RemoveStringFromList PROTO((char *list, char *string));
+int MakeDirIfNeeded PROTO((char *path));
+int IsDvdLoaded PROTO((char *device));
+int StringDuree2Sec( char * string );
 #ifdef __cplusplus
 }
 #endif
