@@ -75,8 +75,6 @@ static const double max_alt_table = 0.00;
 // quant table
 #include "qTable.h"
 
-#include "ac.h"
-
 
 // useful constants
 #define I_TYPE 1
@@ -372,7 +370,7 @@ int old_dc_pred, new_dc_pred;
 		assert(cbuf + x < orbuf + BUF_SIZE); \
 		assert(cbuf + x >= orbuf); \
 		assert(wbuf + x >= orbuf); \
-		tc_memcpy(wbuf, cbuf, x);\
+		memcpy(wbuf, cbuf, x);\
 		cbuf += x; \
 		wbuf += x;
 
@@ -2603,7 +2601,7 @@ int main (int argc, const char * argv[])
 #endif
 
 					// in this case, we'll just use the original slice !
-					tc_memcpy(outTemp, inTemp, cbuf - inTemp);
+					memcpy(outTemp, inTemp, cbuf - inTemp);
 					wbuf = outTemp + (cbuf - inTemp);
 					
 					// adjust outbytecnt
