@@ -82,6 +82,12 @@ COMMENTAIRE :
 #include <setjmp.h>
 
 #include "ac.h"
+#include "globals.h"
+
+int IgnoreErrors=-1;
+
+/* Autocorrection du requant */
+#define USE_CORRECTION_FACTOR		0
 
 // DVD sector size
 #define SECT_SIZE 2048
@@ -139,7 +145,7 @@ int         spu_track_map [32];		// subpicture track# translation map
 int         audio_track_map [8];	// audio track# translation map
 int         verbose;			// level of verbosity
 int         calc_ps_vap;		// calc vaporization based on PS size
-int         preserve;			// preserve audio/spu track numbers
+static int  preserve;			// preserve audio/spu track numbers
 float       vap_fact    = 1.0f;		// vaporization factor from cmd line
 char *injections_file = NULL;		// where to inject internal status from
 const char  progname [] = "vamps";	// we're sucking bytes!
@@ -153,6 +159,9 @@ static int VideoPacketSize;
 static int AudioPacketSize;
 static int SubPacketSize;
 static int NavPacketSize;
+static long long VideoInputSize;
+static long long VideoOutputSize;
+static double RequantCorrectTionFactor = 1;
 
 // injections file stuff
 #define inject_int(v)		{ sizeof (v), &v, "%d"   }
@@ -186,16 +195,17 @@ pthread_cond_t  condw = PTHREAD_COND_INITIALIZER;
 pthread_mutex_t mutr  = PTHREAD_MUTEX_INITIALIZER;
 pthread_mutex_t mutw  = PTHREAD_MUTEX_INITIALIZER;
 
-/*@$#[vaporize.c] static proto. AutoProtoSigV1.1. date: 106/01/24 23:23:11 */
+/*@$#[vaporize.c] static proto. AutoProtoSigV1.1. date: 106/11/09 22:17:47 */
 #include "proto.h"
 #ifdef __cplusplus
 extern "C" {
 #endif
+static void _VarInit PROTO((void));
 static void scan_track_map PROTO((int *map, int ntracks));
 static void flush PROTO((void));
 static void vap_leader PROTO((void));
 static void vap_trailer PROTO((int length));
-static void vaporize PROTO((void));
+static void vaporize PROTO((float fact));
 static void read_injections PROTO((char *filename));
 static void write_injections PROTO((char *filename));
 static void usage PROTO((void));
@@ -208,15 +218,17 @@ static void fatal PROTO((char *fmt, ...));
 // this one lives in requant.c
 extern void *requant_thread (void *);
 extern void requant_initvar( void );
+extern void requant_factorchange();
 
 /*------------------------------------------------------------------------------
 	VAPORIZEINIT-
 Linux!jef 2006/01/24 21:41:53
 ------------------------------------------------------------------------------*/
 
-int VaporizeInit( readFun, writeFun )
+int VaporizeInit( readFun, writeFun, preserveF )
 int (* readFun)();
 int (* writeFun)();
+int preserveF;
 {
 	memset( spu_track_map, 0, sizeof(spu_track_map));		// subpicture track# translation map
 	memset( audio_track_map, 0, sizeof(audio_track_map));		// audio track# translation map
@@ -226,12 +238,13 @@ int (* writeFun)();
 	NoAudioTrack = 1;
 	NoSubpTrack = 1;
 
-	preserve = 1;
+	preserve = preserveF;
 
 	VideoPacketSize = 0;
 	AudioPacketSize = 0;
 	SubPacketSize = 0;
 	NavPacketSize = 0;
+	RequantCorrectTionFactor = 1;
 
 	ReadFun = readFun;
 	WriteFun = writeFun;
@@ -301,7 +314,10 @@ Linux!jef 2006/01/24 21:47:38
 int VaporizeAudio( int audioTrack )
 {
 	if( audioTrack >= 8 )	return( -1 );
-	audio_track_map[audioTrack] = NoAudioTrack++;
+	if( preserve )
+		audio_track_map[audioTrack] = audioTrack +1 ;
+	else
+		audio_track_map[audioTrack] = NoAudioTrack++;
 	return( 0 );
 }
 
@@ -313,7 +329,10 @@ Linux!jef 2006/01/24 21:47:38
 int VaporizeSubp( int subpTrack )
 {
 	if( subpTrack >= 32 )	return( -1 );
-	spu_track_map[subpTrack] = NoSubpTrack++;
+	if( preserve )
+		spu_track_map[subpTrack] = subpTrack + 1;
+	else
+		spu_track_map[subpTrack] = NoSubpTrack++;
 	return( 0 );
 }
 /*------------------------------------------------------------------------------
@@ -351,10 +370,15 @@ int Vaporize( long long size, void * parm )
 {
 	pthread_t            thread;
 	int ret;
+  	int avail;
+	float fact;
 
+	DBG('v',fprintf(stderr, "Vaporize: dynF=%d factor=%f\n", calc_ps_vap, vap_fact););
 	Parm = parm;
 	_VarInit();
 	requant_initvar();
+	VideoInputSize = 0;
+	VideoOutputSize = 0;
 	ps_size = size;
 // read injections file - leaves variables untouched if file not present
 	if( injections_file ) read_injections( injections_file );
@@ -365,49 +389,42 @@ int Vaporize( long long size, void * parm )
 	ret = setjmp( JmpEnv );
 	if( ret )	goto out;
 	ret = 0;
+
+	fact = vap_fact * RequantCorrectTionFactor;
+	DBG('v',fprintf (stderr, "New Requant factor Corrected: %f\n", fact ););
+
 // actually do vaporization
-	vaporize ();
+	vaporize( fact );
 // save internal status for adjacent run
 	if( injections_file )	write_injections (injections_file);
 
-  if( 1)
-  {
-  	int avail;
-
-    fprintf (stderr, "Info: Total bytes read: %llu\n", bytes_read);
-    fprintf (stderr, "Info: Total bytes written: %llu\n", bytes_written);
-    fprintf (stderr, "Info: Total sequence headers in video stream: %d\n",
-	     sequence_headers);
-    fprintf (stderr,
-	     "Info: Total navigation packs (system headers/PCI/DSI): %d\n",
-	     nav_packs);
-    fprintf (stderr,
-"Info: Total bytes vaporized in audio and subpicture streams: %llu (%.2f%%)\n",
-	     (UInt64) skipped_aux_packs * SECT_SIZE,
-	     (double) skipped_aux_packs /
-	     (double) (bytes_read / SECT_SIZE) * 100.0);
-    fprintf (stderr,
-	     "Info: Total bytes vaporized in video stream: %llu (%.2f%%)\n",
-	     (UInt64) skipped_video_packs * SECT_SIZE,
-	     (double) skipped_video_packs /
-	     (double) (bytes_read / SECT_SIZE) * 100.0);
-    fprintf (stderr,
-	     "Info: Total padding bytes in video stream: %llu (%.2f%%)\n",
-	     padding_bytes,
-	     (double) padding_bytes / (double) bytes_written * 100.0);
-    fprintf (stderr, "Info: Total vaporization factor: %.3f\n",
-	     (double) bytes_read / (double) bytes_written);
-    fprintf (stderr,
-	     "Info: Average actual video ES vaporization factor: %.3f\n",
-	     (double) vin_bytes / (double) vout_bytes);
+	DBG('v',fprintf (stderr, "Info: Total bytes read: %llu\n", bytes_read););
+	DBG('v',fprintf (stderr, "Info: Total bytes written: %llu\n", bytes_written););
+	DBG('v',fprintf (stderr, "Info: Total sequence headers in video stream: %d\n",sequence_headers););
+	DBG('v',fprintf (stderr, "Info: Total navigation packs (system headers/PCI/DSI): %d\n", nav_packs););
+	DBG('v',fprintf (stderr, "Info: Total bytes vaporized in audio and subpicture streams: %llu (%.2f%%)\n",(UInt64) skipped_aux_packs * SECT_SIZE,(double) skipped_aux_packs /(double) (bytes_read / SECT_SIZE) * 100.0););
+	DBG('v',fprintf (stderr, "Info: Total bytes vaporized in video stream: %llu (%.2f%%)\n", (UInt64) skipped_video_packs * SECT_SIZE, (double) skipped_video_packs /(double) (bytes_read / SECT_SIZE) * 100.0););
+	DBG('v',fprintf (stderr, "Info: Total padding bytes in video stream: %llu (%.2f%%)\n", padding_bytes, (double) padding_bytes / (double) bytes_written * 100.0););
+	DBG('v',fprintf (stderr, "Info: Total vaporization factor: %.3f\n", (double) bytes_read / (double) bytes_written););
+	DBG('v',fprintf (stderr, "Info: Average actual video ES vaporization factor: %.3f\n", (double) vin_bytes / (double) vout_bytes););
     //fprintf (stderr,
     //	  "total_packs=%d, video_packs=%d, aux_packs=%d, nav_packs=%d\n",
     //	     total_packs, video_packs, aux_packs, nav_packs);
 	avail = rhwp - rptr;
-	if( avail )	fprintf(stderr,"BUG: unread char %d\n", avail );
+	if( avail )	DBG('v',fprintf(stderr,"##BUG: unread char %d\n", avail ););
 	avail = wptr - wbuf;
-	if( avail )	fprintf(stderr,"BUG; unwrite char %d\n", avail );
-  }
+	if( avail )	DBG('v',fprintf(stderr,"##BUG; unwrite char %d\n", avail ););
+
+ /* Compute real video requant factor */
+  	{
+		double realFact = (double)VideoInputSize/(double)VideoOutputSize;
+#if USE_CORRECTION_FACTOR
+		RequantCorrectTionFactor = fact / realFact;
+/* Pour ne pas augmenter la qualite demandee */
+		if( RequantCorrectTionFactor < 1.0 )	RequantCorrectTionFactor = 1.0;
+#endif
+		DBG('v',fprintf(stderr,"VideoInputSize: %lld VideoOutputSize: %lld realFactor: %f Correction: %f\n", VideoInputSize, VideoOutputSize, realFact, RequantCorrectTionFactor ););
+  	}
 out:;
 	pthread_cancel( thread );
 	pthread_join( thread, NULL );
@@ -581,7 +598,7 @@ check_pack (UInt8 *ptr)
   pack_start_code |= (UInt32) (ptr [3]);
 
   if (pack_start_code != 0x000001ba) {
-  	if( pack_start_code == 0 ) {
+  	if( pack_start_code == 0 && IgnoreErrors ) {
 		fprintf(stderr, "Bad pack start code at %llu: %08lx\n", rtell (ptr), pack_start_code);
   		return( -1 ); /* Skip it */
   	}
@@ -672,8 +689,8 @@ check_video_packet (UInt8 *ptr)
 
   // length of video packet plus padding packet must always match sector size
   if (14 + vid_packet_length + pad_packet_length != SECT_SIZE)
-    fatal ("Bad video packet length at %llu: %d",
-	   rtell (ptr), vid_packet_length);
+    fatal ("Bad video packet length at %llu: vpl=%d ppl=%d",
+	   rtell (ptr), vid_packet_length, pad_packet_length );
 
   return rc;
 }
@@ -704,6 +721,8 @@ requant (UInt8 *dst, UInt8 *src, int n, float fact)
   rqt_visize   = (UInt64) ((float) ps_size * (float) vin_bytes /
  			   ((float) total_packs * (float) SECT_SIZE));
 
+  requant_factorchange();
+
   pthread_cond_signal (&condr);
   pthread_mutex_unlock (&mutr);
 
@@ -719,9 +738,7 @@ requant (UInt8 *dst, UInt8 *src, int n, float fact)
 
   pthread_mutex_unlock (&mutw);
 
-  if (verbose >= 2)
-    fprintf (stderr, "Info: Actual video ES vaporization factor: %.3f\n",
-	     (double) n / (double) rv);
+//  DBG('v',fprintf (stderr, "Info: Actual video ES vaporization factor: %.3f\n",(double) n / (double) rv););
 
   return rv;
 }
@@ -771,7 +788,7 @@ new_private_1_type (UInt8 *ptr)
   }
 
   track = audio_track_map [type - abase];
-
+// if( track )	fprintf(stderr, "%s: trackId:0x%x => 0x%x\n", __FUNCTION__, type, track -1 + abase );
   return track ? track - 1 + abase : 0;
 }
 
@@ -818,6 +835,7 @@ new_mpeg_audio_id (int id)
 
   track = audio_track_map [id - 0xc0];
 
+// if( track )	fprintf(stderr, "%s: trackId:0x%x => 0x%x\n", __FUNCTION__, id, track -1 + 0xc0 );
   return track ? track - 1 + 0xc0 : 0;
 }
 
@@ -860,7 +878,7 @@ vap_leader (void)
   while (!lock (SECT_SIZE))
   {
     ptr = rptr;
-    check_pack (ptr);
+    if( check_pack( ptr ) < 0 )	continue;
 
     ptr += 14;
     id   = ptr [3];
@@ -996,7 +1014,7 @@ vap_phase1 (void)
        !lock (seq_length + SECT_SIZE); seq_length += SECT_SIZE)
   {
     ptr = rptr + seq_length;
-    check_pack (ptr);
+    if( check_pack (ptr) < 0 )	continue;
 
     // avoid duplicate counts for sequence headers
     if (seq_length)
@@ -1257,93 +1275,93 @@ vap_phase2 (int seq_length)
 
 // entry point from main()
 // the requant thread already has been started
-static void
-vaporize (void)
+static void vaporize( float fact )
 {
-  int   seq_length;
-  float fact = vap_fact;
+	int   seq_length;
 
-  // process PS up to but not including first sequence header
-  vap_leader ();
+// process PS up to but not including first sequence header
+	vap_leader();
 
-  // just in case - maybe should spit out a warning/error here
-  if (eof)
-    return;
+// just in case - maybe should spit out a warning/error here
+	if (eof)	return;
+//	DBG('v',fprintf(stderr, "Vaporize dynF: %d factor: %f\n", calc_ps_vap, vap_fact););
 
-  total_packs++;
-  nav_packs++;
-  total_packs++;
-  video_packs++;
+	total_packs++;
+	nav_packs++;
+	total_packs++;
+	video_packs++;
 
-  // main loop
-  while (1)
-  {
-    // do phase 1 of vaporization
-    seq_length = vap_phase1 ();
+// main loop
+	while( 1 ) {
+// do phase 1 of vaporization
+		seq_length = vap_phase1 ();
 
-    if (eof)
-    {
-      // EOF on source PS
-      // process packs after and including last sequence header
-      vap_trailer (seq_length);
+		if( eof ) {
+// EOF on source PS
+// process packs after and including last sequence header
+			vap_trailer (seq_length);
 
-      // only exit point from main loop
-      return;
-    }
+// only exit point from main loop
+			return;
+		}
 
-    //fprintf (stderr, "seq_length=%d\n", seq_length);
+//fprintf (stderr, "seq_length=%d\n", seq_length);
 
-    if (calc_ps_vap && vap_fact > 1.0f)
-    {
-      // forecast video ES vaporization factor
-      // the basic formulars look like:
-      // vap_fact  = total_packs/(restpacks+vop)
-      // restpacks = total_packs-(video_packs+skipped_aux_packs)
-      // fact      = (video_packs*net-(gops*net/2+10))/(vop*net-(gops*net/2+10))
-      // net       = SECT_SIZE-(14+9)
-      // 14: pack header size
-      // 9:  PES header size
-      // 10: PTS+DTS size in PES header of sequence header
-      // You are welcome to double check everything here!
-      float vop, net;
-      net  = (float) (SECT_SIZE - (14+9));
-      vop  = video_packs + skipped_aux_packs -
-	     (float) total_packs * (1.0f-1.0f/vap_fact);
-      fact = ((float) video_packs * net -
-	      ((float) sequence_headers * net/2.0f + 10.0f)) /
-	     (vop * net - ((float) sequence_headers * net/2.0f + 10.0f));
+		if( calc_ps_vap && fact > 1.0f) {
+// forecast video ES vaporization factor
+// the basic formulars look like:
+// vap_fact  = total_packs/(restpacks+vop)
+// restpacks = total_packs-(video_packs+skipped_aux_packs)
+// fact      = (video_packs*net-(gops*net/2+10))/(vop*net-(gops*net/2+10))
+// net       = SECT_SIZE-(14+9)
+// 14: pack header size
+// 9:  PES header size
+// 10: PTS+DTS size in PES header of sequence header
+// You are welcome to double check everything here!
+			float vop, net, nfact;
 
-      // requant seems to get stuck on factors < 1
-      if (fact < 1.0f)
-	fact = 1.0f;
+			net  = (float) (SECT_SIZE - (14+9));
+			vop  = video_packs + skipped_aux_packs -
+				(float) total_packs * (1.0f-1.0f/fact);
+			nfact = ((float) video_packs * net -
+				((float) sequence_headers * net/2.0f + 10.0f)) /
+				(vop * net - ((float) sequence_headers * net/2.0f + 10.0f));
 
-	if( verbose >= 2)      fprintf (stderr, "Info: Target video ES vaporization factor: %.3f wanted vap_fact: %f\n", fact, vap_fact);
-    }
+// requant seems to get stuck on factors < 1
+			DBG('v',fprintf (stderr, "Info: Target video ES vaporization factor: %.3f wanted vap_fact: %f\n", nfact, fact););
+/*
+			if( nfact < 1.0f ) {
+				nfact = fact;
+				if( nfact < 1.0f )	nfact = 1.0f;
+			}
+*/
+			if( nfact < fact ) {
+				nfact = fact;
+			}
+			if( nfact < 1.0f )	nfact = 1.0f;
 
-    vin_bytes += vilen;
+			fact = nfact;
+		}
+		vin_bytes += vilen;
+		VideoInputSize += vilen;
+		if (fact > 1.0f) {	// do requantization
+			volen = requant( vobuf, vibuf, vilen, fact );
+		}
+		else {			// don't do requantization
+			tc_memcpy( vobuf, vibuf, vilen );
+			volen = vilen;
+		}
+		vout_bytes += volen;
+		VideoOutputSize += volen;
 
-    if (fact > 1.0f)
-    {
-      // do requantization
-      volen = requant (vobuf, vibuf, vilen, fact);
-    }
-    else
-    {
-      // don't do requantization
-      tc_memcpy (vobuf, vibuf, vilen);
-      volen = vilen;
-    }
+// do phase 2 of vaporization
+		vap_phase2( seq_length );
 
-    vout_bytes += volen;
-
-    // do phase 2 of vaporization
-    vap_phase2 (seq_length);
-
-    //fprintf (stderr,
-    //	       "tot=%d, vid=%d, ps1=%d, nav=%d, sv=%d, sp1=%d, fact=%.3f\n",
-    //	       total_packs, video_packs, aux_packs, nav_packs,
-    //	       skipped_video_packs, skipped_aux_packs, fact);
-  }
+//fprintf (stderr,
+//	       "tot=%d, vid=%d, ps1=%d, nav=%d, sv=%d, sp1=%d, fact=%.3f\n",
+//	       total_packs, video_packs, aux_packs, nav_packs,
+//	       skipped_video_packs, skipped_aux_packs, fact);
+	}
 }
 
 
